#!/usr/bin/env python
#-*- coding:utf-8 -*-

#-----------------------------------------------
#import modules
#-----------------------------------------------

from __future__ import print_function
import netcdf
import os,sys
import scipy
import argparse
import datetime


#-----------------------------------------------
# main function
#-----------------------------------------------

def main(argv):

    # ----------------------------------------
    # parameters
    # ----------------------------------------
    
    ntimestep=1 # new time step in minutes

    parser = argparse.ArgumentParser(description="Transform online trajectories netcdf into lagranto readable netcdf")
    parser.add_argument("inputfile", type=str,help="Input file ")
    parser.add_argument("outputfile", type=str,help="Output file")
    parser.add_argument("ntimestep",type=int,help="new timestep in minutes")
    args = parser.parse_args()
    
    
    filename = args.inputfile
    outname =  args.outputfile
    newtimestep=args.ntimestep

    # ----------------------------------------
    # basic info from initial file
    # ----------------------------------------

    # read attributes
    gattributes=netcdf.read_gattributes(filename)

    # read dimensions
    dimensions=netcdf.read_dimensions(filename)
    
    # read variables
    variables=netcdf.read_variables(filename)

    # read times
    oldtimedim=len(dimensions.values()[1])

    oldtimestep=gattributes['timestep']
    
    # ----------------------------------------
    # Create the new netCDF
    # ----------------------------------------

    # new times
    newtimedim=int(oldtimedim*oldtimestep/60/newtimestep)
    gattributes['duration']=newtimedim*newtimestep
    gattributes['timestep']=newtimestep
    
    index=newtimestep*60/oldtimestep
    
    oldtime=netcdf.read_var(filename,'time')[0]
    newtime=oldtime[1:-1:index]
    refertime=int(newtime[0])
    newtime=(newtime-refertime)
    
    date=datetime.datetime(gattributes['ref_year'],gattributes['ref_month'],gattributes['ref_day'],gattributes['ref_hour'],gattributes['ref_min'])
    date+=datetime.timedelta(seconds=refertime)
    
    gattributes['ref_year']=date.year
    gattributes['ref_month']=date.month
    gattributes['ref_day']=date.day
    gattributes['ref_hour']=date.hour
    gattributes['ref_min']=date.minute
    
    
    # dimensions of the outfile
    ndimensions=[(dimensions.keys()[0],len(dimensions[dimensions.keys()[0]])),(dimensions.keys()[1],newtimedim)]

    # create the file
    ncfile=netcdf.create(outname,ndimensions,gattributes)
    
 
    # add times
    netcdf.addvar(ncfile,'time', newtime,('time'))
    
    # loop over all variables
    for var in variables:
        if (var != 'time') & (var != 'longitude') & (var != 'latitude'):
            vardata=netcdf.read_var(filename,var)[0]
            netcdf.addvar(ncfile,var,vardata[1:-1:index,:],('time','id'))
        elif (var == 'longitude') & (gattributes['pollat'] != 90):
            londata=netcdf.read_var(filename,'longitude')[0]
            latdata=netcdf.read_var(filename,'latitude')[0]
            nlon,nlat=netcdf.rotcoo(londata,latdata,'r2n',lonpol=gattributes['pollon'],latpol=gattributes['pollat'])
            netcdf.addvar(ncfile,'longitude',nlon[1:-1:index,:],('time','id'))
            netcdf.addvar(ncfile,'latitude',nlat[1:-1:index,:],('time','id'))
    
if __name__ == '__main__':
	main(sys.argv[1:])


