c***********************************************************************
c CF-netCDF / Reading
c***********************************************************************

c     -------------------------------------------------------------------
c     cdfopn
c     -------------------------------------------------------------------

      subroutine cdfopn_mch (filnam,cdfid,ierror)

      use netcdf
      
      character*(*) filnam
      integer 	    cdfid
      integer       ierror

      ierror = NF90_OPEN(TRIM(filnam),nf90_nowrite, cdfid)
      IF ( ierror /= nf90_NoErr ) PRINT *,NF90_STRERROR(ierror)

      return
      end


c     -------------------------------------------------------------------
c     clscdf
c     -------------------------------------------------------------------

      subroutine clscdf_mch (cdfid, ierror)

      use netcdf
      
      integer      cdfid
      integer      ierror

      ierror = NF90_CLOSE(cdfid)
      IF( ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      end


c     -------------------------------------------------------------------
c     getdef
c     -------------------------------------------------------------------

      subroutine getdef_mch (cdfid, varnam, ndim, misdat, 
     &                      vardim, varmin, varmax, stag, ierror)

      use netcdf

      integer        cdfid
      character *(*) varnam
      integer        ndim
      real           misdat
      integer        vardim(4)
      real           varmin(4)
      real           varmax(4)
      real           stag(4)
      integer        ierror

      real           lat(5000)
      real           lon(5000)
      real           lev(5000)
      real           tim(5000)
      real           dlon,dlat
      integer        varid
      integer        lonid
      integer        latid
      integer        levid
      integer        timid
      integer        dimids (nf90_max_var_dims)
      character*80   dimname(nf90_max_var_dims)
      integer        nlon
      integer        nlat
      integer        nlev
      integer        ntim
      character*(80) attname
      integer        i

c     Get <ndim>, <dimids> and <dimname>
      ierror = NF90_INQ_VARID(cdfid,varnam,varid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_variable(cdfid, varid, ndims  = ndim)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_variable(cdfid, varid, 
     >                                   dimids = dimids(1:ndim))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      do i=1,ndim
         ierror = nf90_inquire_dimension(cdfid, dimids(i), 
     >                               name = dimname(i) )
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      enddo

c     Check which dimensions are available
      lonid = -999
      latid = -999
      levid = -999
      timid = -999
      do i=1,ndim
        if ( dimname(i).eq.'x_1' ) then
           ierror = nf90_inq_dimid(cdfid,dimname(i), lonid)
           IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        endif
        if ( dimname(i).eq.'y_1' ) then
           ierror = nf90_inq_dimid(cdfid,dimname(i), latid)
           IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        endif
        if ( ( dimname(i).eq.'z_1' ).or.
     >       ( dimname(i).eq.'z_2' ).or.
     >       ( dimname(i).eq.'z_3' ).or.
     >       ( dimname(i).eq.'z_4' ).or.
     >       ( dimname(i).eq.'z_5' ).or.
     >       ( dimname(i).eq.'z_6' ) ) 
     >  then
           ierror = nf90_inq_dimid(cdfid,dimname(i), levid)
           IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        endif
        if ( dimname(i).eq.'time' ) then
           ierror = nf90_inq_dimid(cdfid,'time', timid)
           IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        endif
      enddo  
 
c     Read lon and set <vardim(1)>, <varmin(1)>, <varmax(1)>, <stag(1)>
      if ( lonid.ne.-999 ) then
        ierror = nf90_inquire_dimension(cdfid, lonid, len = nlon)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_INQ_VARID(cdfid,'x_1',varid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_GET_VAR(cdfid,varid,lon(1:nlon))
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
        vardim(1) = nlon
        varmin(1) = lon( 1    )
        varmax(1) = lon( nlon )
        stag  (1) = 0.
      else
        vardim(1) = -999
        varmin(1) = 0.
        varmax(1) = 0.
        stag  (1) = 0.
      endif
      
c     Read lat and set <vardim(2)>, <varmin(2)>, <varmax(2)>, <stag(2)>
      if ( latid.ne.-999 ) then
        ierror = nf90_inquire_dimension(cdfid, latid, len = nlat)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_INQ_VARID(cdfid,'y_1',varid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_GET_VAR(cdfid,varid,lat(1:nlat))
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
        vardim(2) = nlat
        varmin(2) = lat( 1    )
        varmax(2) = lat( nlat )
        stag  (2) = 0.
      else
        vardim(2) = -999
        varmin(2) = 0.
        varmax(2) = 0.
        stag  (2) = 0.
      endif
      
c     Read lev and set <vardim(3)>, <varmin(3)>, <varmax(3)>, <stag(3)>
      if ( levid.ne.-999 ) then
        ierror = nf90_inquire_dimension(cdfid, levid, len = nlev)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        vardim(3) = nlev
        varmin(3) = 1
        varmax(3) = nlev
        stag  (3) = 0.
        if ( (trim(varnam) == 'W'  ) .OR. 
     >       (trim(varnam) == 'HHL') .OR.
     >       (trim(varnam) == 'TKE' ))
     >  then
             stag(3) = -0.5
        endif
      else
        vardim(3) = -999
        varmin(3) = 0.
        varmax(3) = 0.
        stag  (3) = 0.
      endif

c     Read time and set <vardim(4)>, <varmin(4)>, <varmax(4)>, <stag(4)>
      if ( timid.ne.-999 ) then
        ierror = nf90_inquire_dimension(cdfid, timid, len = ntim)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_INQ_VARID(cdfid,'time',varid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_GET_VAR(cdfid,varid,tim(1:ntim))
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
        vardim(4) = ntim
        varmin(4) = tim( 1    )
        varmax(4) = tim( ntim )
        stag  (4) = 0.0
      else
        vardim(4) = -999
        varmin(4) = 0.
        varmax(4) = 0.
        stag  (4) = 0.
      endif  

c     Read missing data value <misdat>
      ierror = NF90_INQ_VARID(cdfid,varnam,varid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_get_att(cdfid, varid, "_FillValue", misdat)
      IF(ierror /= nf90_NoErr) THEN
         misdat = -999.99
         ierror = 0
      ENDIF

c     Do some checks
      if ( varmax(1).lt.varmin(1) ) then
         print*,' Error: Wrong order of longitudes'
         ierror = 1
         return
      endif
      if ( varmax(2).lt.varmin(2) ) then
         print*,' Error: Wrong order of latitudes'
         ierror = 1
         return
      endif

      end

c     -------------------------------------------------------------------
c     getgrid
c     -------------------------------------------------------------------

      subroutine getgrid_mch (cdfid,dx,dy,ierror)

      use netcdf

      integer   cdfid
      real      dx
      real      dy
      integer   ierror

      integer   lonid
      integer   latid
      real      lat(5000)
      real      lon(5000)
      integer   nlon
      integer   nlat

c     Get the ID for <longitude>, <latitude>, <level> and <time>
      ierror = nf90_inq_dimid(cdfid, 'x_1', lonid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, lonid, len = nlon)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_INQ_VARID(cdfid, 'x_1' ,lonid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inq_dimid(cdfid, 'y_1', latid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, latid, len = nlat)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_INQ_VARID(cdfid, 'y_1' ,latid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Read lon and calculate <dx>
      ierror = NF90_GET_VAR(cdfid,lonid,lon(1:nlon))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      dx = ( lon(nlon) - lon(1) ) / ( real(nlon) - 1.)

c     Read lat and calculate <dy>
      ierror = NF90_GET_VAR(cdfid,latid,lat(1:nlat))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
      dy = ( lat(nlat) - lat(1) ) / ( real(nlat) - 1.)

      end


c     -------------------------------------------------------------------
c     getpole
c     -------------------------------------------------------------------

      subroutine getpole_mch (cdfid,pollon,pollat,polgam,ierror)

      use netcdf

      integer   cdfid, varid
      real      pollon, pollat, polgam
      integer   ierror

      ierror = nf90_inq_varid(cdfid, "grid_mapping_1", varid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      
      ierror  = nf90_get_att(cdfid, varid,
     >                       "grid_north_pole_longitude", pollon)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_get_att(cdfid, varid,
     >                       "grid_north_pole_latitude", pollat)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_get_att(cdfid, varid,
     >                       "north_pole_grid_longitude", polgam)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      end

c     -------------------------------------------------------------------
c     getcfn
c     -------------------------------------------------------------------

      subroutine getcfn_mch (cdfid,cfn,ierror)
      
      use netcdf

      integer      cdfid
      character*80 cfn
      integer      ierror

      cfn='LMCONSTANTS'

      end


c     -------------------------------------------------------------------
c     getdat
c     -------------------------------------------------------------------

      subroutine getdat_mch (cdfid, varnam, time, level, dat, ierror)

      use netcdf

      integer       cdfid
      character*(*) varnam
      real          time
      integer       level
      real          dat(*)
      integer       ierror

      integer       varid
      integer       ndim
      real          misdat
      integer       vardim(4)
      real          varmin(4)
      real          varmax(4)
      real          stag(4)
      integer       ntim
      real          tim(5000)
      integer       itime
      integer       corner(4)
      integer       count (4)
      real,allocatable, dimension (:,:,:) :: tmp3
      integer       stat
      integer       i,j,k
      character*80  dir

c     Get variable definition
      call getdef_mch (cdfid, varnam, ndim, misdat, 
     &                vardim, varmin, varmax, stag, ierror)

c     Get list of times and get appropriate time index
      if ( vardim(4).gt.0 ) then
         call gettimes_mch (cdfid,tim,ntim,ierror)
         itime = 0
         do i=1,ntim
            if ( tim(i).eq.time) itime = i
         enddo
         if ( itime.eq.0 ) then
            ierror = 1
            print*,'Error: Unknown time in getdat'
            return
         endif
      else 
         itime  = 1
         ntim   = 0
      endif

c     Select box in netCDF file
      count(1) = -999
      count(2) = -999
      count(3) = -999
      count(4) = -999
      if ( vardim(1).gt.0 ) then
         corner(1) = 1
         count (1) = vardim(1)
      else
         print*,' ERROR: lon missing for ',trim(varnam)
         stop
      endif
      if ( vardim(2).gt.0 ) then
         corner(2) = 1
         count (2) = vardim(2)
      else
         print*,' ERROR: lat missing for ',trim(varnam)
         stop
      endif
      if ( (vardim(3).gt.0).and.(level.eq.0) ) then
         corner(3) = 1
         count (3) = vardim(3)
      elseif ( (vardim(3).gt.0).and.(level.ge.1) ) then
         corner(3) = vardim(3) - level + 1
         count (3) = 1
      endif
      if ( vardim(4).gt.0 ) then
         corner(4) = itime
         count (4) = 1
      endif
          
c     Read data
      ierror = NF90_INQ_VARID(cdfid,varnam,varid)
      IF (ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      if ( (vardim(3).gt.0).and.(vardim(4).gt.0) ) then
        allocate(tmp3(count(1),count(2),count(3)),stat=stat)
        ierror = nf90_get_var(cdfid,varid,tmp3,
     >           start = (/ corner(1),corner(2),corner(3),corner(4) /),
     >           count = (/ count (1),count (2),count (3),count (4) /) ) 
     
      elseif ( (vardim(3).lt.0).and.(vardim(4).gt.0) ) then
        allocate(tmp3(count(1),count(2),1),stat=stat)
        ierror = nf90_get_var(cdfid,varid,tmp3,
     >           start = (/ corner(1),corner(2),corner(4) /),
     >           count = (/ count (1),count (2),count (4) /) ) 
     
      elseif ( (vardim(3).gt.0).and.(vardim(4).lt.0) ) then
        allocate(tmp3(count(1),count(2),count(3)),stat=stat)
        ierror = nf90_get_var(cdfid,varid,tmp3,
     >           start = (/ corner(1),corner(2),corner(3) /),
     >           count = (/ count (1),count (2),count (3) /) ) 
      
      elseif ( (vardim(3).lt.0).and.(vardim(4).lt.0) ) then
        allocate(tmp3(count(1),count(2),1),stat=stat)
        ierror = nf90_get_var(cdfid,varid,tmp3,
     >           start = (/ corner(1),corner(2) /),
     >           count = (/ count (1),count (2) /) ) 
     
      endif

c     Copy data to output array: change orientation of vertical axis
      do i=1,count(1)
         do j=1,count(2)
            if ( count(3).gt.1) then

               do k=1,count(3)
                     dat(i+count(1)*(j-1)+count(1)*count(2)*(k-1)) = 
     >                                      tmp3(i,j,count(3)-k+1)
               enddo

            else 
               dat(i+count(1)*(j-1)) = tmp3(i,j,1)
            endif

         enddo
      enddo
               
c     Deallocate memory
      deallocate(tmp3,stat=stat)

      end

c     -------------------------------------------------------------------
c     gettimes
c     -------------------------------------------------------------------

      subroutine gettimes_mch (cdfid,times,ntimes,ierror)

      use netcdf
      
      integer cdfid
      real    times(*)
      integer ntimes
      integer ierror
      
      integer timid

c     Get all times 
      ierror = nf90_inq_dimid(cdfid,'time', timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, timid, len = ntimes)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_INQ_VARID(cdfid,'time',timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_GET_VAR(cdfid,timid,times(1:ntimes))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end


c     -------------------------------------------------------------------
c     getvars
c     -------------------------------------------------------------------

      subroutine getvars_mch (cdfid,nvars,vnam,ierror)

      use netcdf
      
      integer           cdfid
      integer           nvars
      character*80      vnam(*)
      integer           ierror

      integer           i

c     Get number of variables
      ierror = nf90_inquire(cdfid, nVariables = nvars)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      
c     Get variable names
      do i=1,nvars
          ierror = nf90_inquire_variable(cdfid, i, name = vnam(i) )
          IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      enddo

      return
      end


c***********************************************************************
c CF-netCDF / Writing
c***********************************************************************

c     -------------------------------------------------------------------
c     crecdf
c     -------------------------------------------------------------------

      subroutine crecdf_mch (filnam, cdfid, phymin, phymax, ndim, 
     &                      nlon, nlat, nlev, pollon, pollat, ierror) 
      
      use netcdf
      
      character*(* ) filnam
      integer        cdfid
      real           phymin(4), phymax(4)
      real           pollon, pollat
      real           val(1000)
      integer        ndim, dimid(4), varid(4)
      integer        nlon, nlat, nlev
      integer        ierror, i

c     Create the netCDF file
      ierror = nf90_create(path  = filnam, 
     >                     cmode = nf90_clobber, 
     >                     ncid  = cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Global attributes
      ierror = nf90_put_att(cdfid, nf90_global, 
     >                            "Conventions", "CF")
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, nf90_global, 
     >                             "institution", 'IACETH')
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     longitude dimension
      ierror = nf90_def_dim(cdfid, "rlon", nlon, dimid(1))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_def_var(cdfid, "rlon",
     >                      NF90_FLOAT,dimid(1),varid(1))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(1), 
     >                      "long_name", "rotated longitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(1), 
     >                      "units", "degrees") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(1), 
     >                      "standard_name", "longitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     latitude dimension
      ierror = nf90_def_dim(cdfid, "rlat", nlat, dimid(2))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_def_var(cdfid, "rlat",
     >                      NF90_FLOAT,dimid(2),varid(2))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(2), 
     >                      "long_name", "rotated latitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(2), 
     >                      "units", "degrees") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(2), 
     >                      "standard_name", "latitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     level dimension
      ierror = nf90_def_dim(cdfid, "level", nlev, dimid(3))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_def_var(cdfid, "level",
     >                      NF90_FLOAT,dimid(3),varid(3))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(3), 
     >                      "long_name", "hybrid vertical level") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(3), 
     >                      "units", "level") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     rotated pole
      ierror = nf90_def_var(cdfid, "rotated_pole", NF90_CHAR,
     >                      varid=varid(4))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(4), "long_name",
     >     "coordinates of the rotated north pole")
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(4),
     >     "grid_north_pole_longitude", pollon)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(4),
     >     "grid_north_pole_latitude", pollat)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     End definition mode
      ierror = nf90_enddef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     write longitude, latitude and level
      do i=1,nlon
         val(i) = phymin(1) + real(i-1) *
     >            ( phymax(1) - phymin(1) ) / real(nlon-1)
      enddo
      ierror = nf90_put_var(cdfid, varid(1), val(1:nlon))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      do i=1,nlat
         val(i) = phymin(2) + real(i-1) *
     >            ( phymax(2) - phymin(2) ) / real(nlat-1)
      enddo
      ierror = nf90_put_var(cdfid, varid(2), val(1:nlat))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      do i=1,nlev
         val(i) = real(i) - 1.
      enddo
      ierror = nf90_put_var(cdfid, varid(3), val(1:nlev))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      end


c     -------------------------------------------------------------------
c     putdef
c     -------------------------------------------------------------------

      subroutine putdef_mch (cdfid, newvar, ndimension, misdat, 
     &                      vardim, varmin, varmax, stag, ierror)

      use netcdf
      
      integer        cdfid
      character*(*)  newvar
      integer        ndimension
      real           misdat
      integer        vardim(4)
      real           varmin(4)
      real           varmax(4)
      real           stag(4)
      integer        ierror

      real          eps
      parameter     (eps=0.0001)
      
      integer       ndim
      integer       nvar
      character*80  dimname(100)
      character*80  varname(100)
      integer       dimid
      integer       dimids(4)
      integer       nval
      real          val(1000)
      integer       varid
      integer       varids(4)
      integer       writelon,writelat,writelev,writetim
      character*80  lonname,latname,levname,timname

c     Unit/name table
      character*80  tablefile
      parameter     (tablefile='nil')
      character*80  table_varname (1000)
      character*80  table_longname(1000)
      character*80  table_unit    (1000)
      integer       fid
      integer       ntable

c     Read table information
      if ( tablefile.ne.'nil' ) then
         ntable = 0
         fid    = 10
         open(fid,file=tablefile)
 100      ntable = ntable + 1
          read(fid,*) table_varname (ntable),
     >                        table_longname(ntable),
     >                        table_unit    (ntable)

          print*,table_varname(ntable),table_longname(ntable),
     >             table_unit(ntable)
          
          goto 100
 101     close(fid)
      endif

c     Set the default names for dimensions
      lonname = 'rlon'
      latname = 'rlat'
      levname = 'level'
      timname = 'time'

c     Get number of dimensions and of variables
      ierror = nf90_inquire(cdfid, nDimensions    = ndim,
     >                             nVariables     = nvar )
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Get a list of all dimensions
      if ( ndim.gt.0 ) then
         do i=1,ndim
            ierror = nf90_inquire_dimension(cdfid,i,name = dimname(i))
            IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         enddo
      endif

c     Get a list of all variables
      if ( nvar.gt.0 ) then
         do i=1,nvar
            ierror = nf90_inquire_variable(cdfid,i,name = varname(i))
            IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         enddo
      endif

c     Check whether the new variable name is in conflict with the others
      do i=1,nvar
         if ( newvar.eq.varname(i) ) then
            print*,trim(newvar),' already exists on the file'
            stop
         endif
      enddo

c     Set IDs for dimensions 
      do i=1,4
         dimids(i) = -1
      enddo
      writelon = 0
      writelat = 0
      writelev = 0
      writetim = 0

c     Check whether existing dimensions can be used
      do i=1,ndim

c        Read dimension values
         ierror = nf90_inquire_dimension(cdfid,i,len = nval)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)         
         ierror = NF90_INQ_VARID(cdfid,dimname(i),varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
         ierror = NF90_GET_VAR(cdfid,varid,val(1:nval))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

c        Check whether it can be used for the first dimension (lon)
         if ( (vardim(1).eq.nval).and.
     >        (abs(varmin(1)-val(1)   ).lt.eps).and.
     >        (abs(varmax(1)-val(nval)).lt.eps) )
     >   then
            dimids(1) = i
         endif

c        Check whether it can be used for the second dimension (lat)
         if ( (vardim(2).eq.nval).and.
     >        (abs(varmin(2)-val(1)   ).lt.eps).and.
     >        (abs(varmax(2)-val(nval)).lt.eps) )
     >   then
            dimids(2) = i
         endif
         
c        Check whether it can be used for the third dimension (lev)
         if ( (vardim(3).eq.nval) ) then
            dimids(3) = i
         endif
         
c        Check whether there is already a time dimension (tim)
         ierror = nf90_inquire(cdfid, unlimitedDimId = dimids(4) )
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      enddo

c     Decide what dimension names to use
      if ( dimids(1).eq.-1 ) then
         do i=1,ndim
            if ( dimname(i).eq.lonname ) then
               lonname = trim(lonname)//'_'//trim(newvar)
            endif
         enddo
      endif
      if ( dimids(2).eq.-1 ) then
         do i=1,ndim
            if ( dimname(i).eq.latname ) then
               latname = trim(latname)//'_'//trim(newvar)
            endif
         enddo
      endif
      if ( dimids(3).eq.-1 ) then
         do i=1,ndim
            if ( dimname(i).eq.levname ) then
               levname = trim(levname)//'_'//trim(newvar)
            endif
         enddo
      endif

c     Enter definition mode
      ierror = nf90_redef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Define dimensions and corresponding variables if necessary
      if ( dimids(1).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid, lonname, vardim(1), dimids(1))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, lonname,
     >                                 NF90_FLOAT,dimids(1),varids(1))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(1), 
     >        "long_name", "rotated longitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(1), 
     >        "units", "degrees") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(1), 
     >        "standard_name", "longitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         writelon = 1
      endif
      if ( dimids(2).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid, latname, vardim(2), dimids(2))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, latname,
     >                                NF90_FLOAT,dimids(2),varids(2))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(2), 
     >        "long_name", "rotated latitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(2), 
     >        "units", "degrees") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(2), 
     >        "standard_name", "latitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         writelat = 1
      endif
      if ( dimids(3).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid, levname, vardim(3), dimids(3))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, levname,
     >                                NF90_FLOAT,dimids(3),varids(3))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(3), 
     >        "long_name", "hybrid vertical level")
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(3), 
     >        "units", "level") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         writelev = 1
      endif
      if ( dimids(4).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid,timname, nf90_unlimited, dimids(4))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, timname,
     >                                NF90_FLOAT,dimids(4),varids(4))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(4), 
     >                 "units", "hours") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(4), 
     >                 "calendar", "proleptic_gregorian") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         
         writetim = 1
      endif

c     Define the variable
      ierror = nf90_def_var(cdfid,newvar,
     >                      NF90_FLOAT,dimids,varid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Write variable attributes
      ierror = nf90_put_att(cdfid, varid, 
     >                 "long_name", "unknown (please add with NCO)") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid, 
     >                 "units", "unknown (please add with NCO)") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid, 
     >                 "_FillValue", real(-999.99) ) 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)


c     Leave definition mode
      ierror = nf90_enddef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Write dimension arrays
      if ( writelon.eq.1 ) then
         do i=1,vardim(1)
            val(i) = varmin(1) + real(i-1) *
     >                 ( varmax(1) - varmin(1) ) / real(vardim(1)-1)
         enddo
         ierror = nf90_inq_varid(cdfid,lonname,varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_var(cdfid, varid, val(1:vardim(1)))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      endif
      if ( writelat.eq.1 ) then
         do i=1,vardim(2)
            val(i) = varmin(2) + real(i-1) *
     >                 ( varmax(2) - varmin(2) ) / real(vardim(2)-1)
         enddo
         ierror = nf90_inq_varid(cdfid,latname,varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_var(cdfid, varid, val(1:vardim(2)))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      endif

      if ( writelev.eq.1 ) then
         do i=1,vardim(3)
            val(i) = real(i)
         enddo
         ierror = nf90_inq_varid(cdfid,levname,varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_var(cdfid, varid, val(1:vardim(3)) )
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      endif

      end


c     -------------------------------------------------------------------
c     puttimes
c     -------------------------------------------------------------------

      subroutine puttimes_mch (cdfid,times,ntimes,ierror)

      use netcdf
      
      integer   cdfid
      real      times(*)
      integer   ntimes
      integer   ierror

      integer   timid
      integer   otimes

c     Get number of times 
      ierror = nf90_inq_dimid(cdfid,'time', timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, timid, len = otimes)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      
      if (otimes.lt.ntimes) then
         print *,'Warning: puttimes is extending times-array'
      else if (nfiltim.gt.ntimes) then
         print *,'Warning: puttimes does not cover range of times-array'
      endif

c     Write times
      ierror = NF90_INQ_VARID(cdfid,'time',timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_PUT_VAR(cdfid,timid,times(1:ntimes))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

c     Sync
      ierror = nf90_sync(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end


c     -------------------------------------------------------------------
c     putstart
c     -------------------------------------------------------------------

      subroutine putstart_mch (cdfid,idate,ierror)

      use netcdf

      integer      cdfid
      integer      idate(5)
      integer      ierror

c     Enter definition mode
      ierror = nf90_redef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Write data      
      ierror  = nf90_put_att(cdfid,nf90_global, "starty", idate(1))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "startm", idate(2))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "startd", idate(3))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "starth", idate(4))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "starts", idate(5))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Exit definition mode
      ierror = nf90_enddef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Sync
      ierror = nf90_sync(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end


c     -------------------------------------------------------------------
c     cdfwopn
c     -------------------------------------------------------------------

      subroutine cdfwopn_mch (filnam,cdfid,ierror)
      
      use netcdf
      
      character*(*) filnam
      integer       cdfid
      integer       ierror

      ierror = NF90_OPEN(TRIM(filnam),nf90_write, cdfid)
      IF ( ierror /= nf90_NoErr ) PRINT *,NF90_STRERROR(ierror)

      return
      end


c     -------------------------------------------------------------------
c     putdat
c     -------------------------------------------------------------------

      subroutine putdat_mch (cdfid, varnam, time, level, dat, ierror)

      use netcdf
      
      integer        cdfid
      character*(*)  varnam
      real           time
      integer        level
      real           dat(*)
      integer        ierror

      real           eps
      parameter      (eps=0.0001)

      integer        ndims
      real           misdat
      integer        vardim(4)
      real           varmin(4)
      real           varmax(4)
      real           stag(4)
      integer        varid
      integer        ntimes
      real           times(100)
      integer        itime
      integer        corner(4)
      integer        count(4)
      integer        stat
      real,allocatable, dimension (:,:,:,:) :: tmp

c     Get definitions of data
      call getdef_cf (cdfid, trim(varnam), ndims, misdat, 
     &                vardim, varmin, varmax, stag, ierr)
      if (ierr.ne.0)  print *,'*ERROR* in getdef in putdat'

c     Get id of the variable
      ierror = NF90_INQ_VARID(cdfid,varnam,varid)
      IF (ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Get a list of all times 
      call gettimes_cf(cdfid,times,ntimes,ierror)

c     Get the time index
      itime = -1
      do i=1,ntimes
         if ( abs(time-times(i)).lt.eps) itime = i
      enddo

c     Start a new time step, if necessary
      if ( itime.eq.-1 ) then
         ntimes        = ntimes + 1
         times(ntimes) = time
         itime         = ntimes
         call puttimes_cf(cdfid,times,ntimes,ierror)
      endif

C     Define data volume to write on the NetCDF file in index space
      corner(1) = 1               
      corner(2) = 1
      count(1)  = vardim(1)       
      count(2)  = vardim(2)
      if (level.eq.0) then
        corner(3) = 1
        count(3)  = vardim(3)
      else
        corner(3) = vardim(3)-level+1
        count(3)  = 1
      endif
      corner(4) = itime
      count(4)  = 1

c     Allocate temporary array
      allocate(tmp(count(1),count(2),count(3),count(4)),stat=stat)

c     Copy data to output array: change orientation of vertical axis
      if ( count(3).gt.1 ) then
         do i=1,count(1)
            do j=1,count(2)
               do k=1,count(3)
                  tmp(i,j,count(3)-k+1,1) =
     >               dat(i+count(1)*(j-1)+count(1)*count(2)*(k-1)) 
               enddo
            enddo
         enddo
      else
         do i=1,count(1)
            do j=1,count(2)
               tmp(i,j,1,1) = dat(i+count(1)*(j-1)) 
            enddo
         enddo
      endif

c    Re-scale specific variables
      if ((trim(varnam) == 'QV') .or. (trim(varnam) == 'QC') .or.
     >    (trim(varnam) == 'QI') .or. (trim(varnam) == 'QR') .or.
     >    (trim(varnam) == 'QS') .or. (trim(varnam) == 'QG') .or.
     >    (trim(varnam) == 'Q_SEDIM') .or.
     >    (trim(varnam) == 'QV_2M')) 
     >then
           tmp = tmp / 1000.
           print*,' WARNING: re-scaling ',trim(varnam),' 1/1000'
      endif 
      if ((trim(varnam) == 'P') .or. (trim(varnam) == 'PS') .or.
     >    (trim(varnam) == 'PMSL') .or. (trim(varnam) == 'P_Z'))
     >then
           tmp = tmp / 0.01
           print*,' WARNING: re-scaling ',trim(varnam),' 1/0.01'
      endif
      if ((trim(varnam) == 'T') .or. (trim(varnam) == 'T_2M') .or.
     >    (trim(varnam) == 'TD_2M') .or.
     >    (trim(varnam) == 'TMIN_2M') .or.
     >    (trim(varnam) == 'TMAX_2M') .or.
     >    (trim(varnam) == 'T_P') .or. (trim(varnam) == 'T_Z'))
     >then
         tmp = tmp + 273.15
         print*,' WARNING: re-scaling ',trim(varnam),' + 273.15'
      endif

c     Write data
      ierror = nf90_put_var(cdfid, varid, tmp,
     >           start = (/ corner(1),corner(2),corner(3),corner(4) /),
     >           count = (/ count (1),count (2),count (3),count (4) /) ) 

c     Deallocate memory
      deallocate(tmp,stat=stat)

c     Sync
      ierror = nf90_sync(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end

 




 
c***********************************************************************
c CF-netCDF / Reading
c***********************************************************************

c     -------------------------------------------------------------------
c     cdfopn
c     -------------------------------------------------------------------

      subroutine cdfopn_cf(filnam,cdfid,ierror)

      use netcdf
      
      character*(*) filnam
      integer 	    cdfid
      integer       ierror

      ierror = NF90_OPEN(TRIM(filnam),nf90_nowrite, cdfid)
      IF ( ierror /= nf90_NoErr ) PRINT *,NF90_STRERROR(ierror)

      return
      end


c     -------------------------------------------------------------------
c     clscdf
c     -------------------------------------------------------------------

      subroutine clscdf_cf (cdfid, ierror)

      use netcdf
      
      integer      cdfid
      integer      ierror

      ierror = NF90_CLOSE(cdfid)
      IF( ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      end


c     -------------------------------------------------------------------
c     getdef
c     -------------------------------------------------------------------

      subroutine getdef_cf (cdfid, varnam, ndim, misdat, 
     &                      vardim, varmin, varmax, stag, ierror)

      use netcdf

      integer        cdfid
      character *(*) varnam
      integer        ndim
      real           misdat
      integer        vardim(4)
      real           varmin(4)
      real           varmax(4)
      real           stag(4)
      integer        ierror

      real           lat(5000)
      real           lon(5000)
      real           lev(5000)
      real           tim(5000)
      real           dlon,dlat
      integer        varid
      integer        lonid
      integer        latid
      integer        timid
      integer        dimids (nf90_max_var_dims)
      character*80   dimname(nf90_max_var_dims)
      integer        nlon
      integer        nlat
      integer        nlev
      integer        ntim
      character*(80) attname
      integer        i

c     Get <varid>
      ierror = NF90_INQ_VARID(cdfid,varnam,varid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Get <ndim>, <dimids> and <dimname>
      ierror = nf90_inquire_variable(cdfid, varid, ndims  = ndim)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_variable(cdfid, varid, 
     >                                   dimids = dimids(1:ndim))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      do i=1,ndim
         ierror = nf90_inquire_dimension(cdfid, dimids(i), 
     >                               name = dimname(i) )
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      enddo

      if ( (ndim.ne.3).and.(ndim.ne.4) ) then
	 print*,' ERROR: getdef only support 3d and 4d arraysd'
         print*,' 3D=time,lat,lon; 4D=time,level,lat,lon'
	 stop
      endif

c     Get the ID for <longitude>, <latitude>, <level> and <time>
      if (ndim.eq.4) then
        ierror = nf90_inq_dimid(cdfid,dimname(1), lonid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = nf90_inq_dimid(cdfid,dimname(2), latid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = nf90_inq_dimid(cdfid,dimname(3), levid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = nf90_inq_dimid(cdfid,dimname(4), timid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      else if ( ndim.eq.3 ) then
        ierror = nf90_inq_dimid(cdfid,dimname(1), lonid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = nf90_inq_dimid(cdfid,dimname(2), latid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = nf90_inq_dimid(cdfid,dimname(3), timid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      endif

c     Read lon and set <vardim(1)>, <varmin(1)>, <varmax(1)>, <stag(1)>
      ierror = nf90_inquire_dimension(cdfid, lonid, len = nlon)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_INQ_VARID(cdfid,dimname(1),lonid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_GET_VAR(cdfid,lonid,lon(1:nlon))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
      vardim(1) = nlon
      varmin(1) = lon( 1    )
      varmax(1) = lon( nlon )
      stag  (1) = 0.
      if (trim(dimname(1)) == 'srlon') then
         dlon      = lon(2) - lon(1)
         varmin(1) = varmin(1) - 0.5*dlon
         varmax(1) = varmax(1) - 0.5*dlon
      endif

c     Read lat and set <vardim2(2)>, <varmin(2)>, <varmax(2)>, <stag(2)>
      ierror = nf90_inquire_dimension(cdfid, latid, len = nlat)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_INQ_VARID(cdfid,dimname(2),latid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_GET_VAR(cdfid,latid,lat(1:nlat))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
      vardim(2) = nlat
      varmin(2) = lat( 1    )
      varmax(2) = lat( nlat )
      stag  (2) = 0.
      if (trim(dimname(2)) == 'srlat') then
         dlat      = lat(2) - lat(1)
         varmin(2) = varmin(2) - 0.5*dlat
         varmax(2) = varmax(2) - 0.5*dlat
      endif
      
c     Read lev and set <vardim2(3)>, <varmin(3)>, <varmax(3)>, <stag(3)>
      if ( ndim.eq.4 ) then
        ierror = nf90_inquire_dimension(cdfid, levid, len = nlev)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        vardim(3) = nlev
        varmin(3) = 1
        varmax(3) = nlev
        stag  (3) = 0.
        if ( (trim(varnam) == 'W'  ) .OR. 
     >       (trim(varnam) == 'HHL') .OR.
     >       (trim(varnam) == 'TKE' ))
     >       stag(3) = -0.5
      else if (ndim.eq.3) then
        vardim(3) = 1
        varmin(3) = 1
        varmax(3) = 1
        stag  (3) = 0.
      endif 

c     Read time and set <vardim2(4)>, <varmin(4)>, <varmax(4)>, <stag(4)>
      if ( ndim.eq.4 ) then
        ierror = nf90_inquire_dimension(cdfid, timid, len = ntim)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_INQ_VARID(cdfid,dimname(4),timid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_GET_VAR(cdfid,timid,tim(1:ntim))
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
        vardim(4) = ntim
        varmin(4) = tim( 1    )
        varmax(4) = tim( ntim )
        stag  (4) = 0.0
      else if ( ndim.eq.3 ) then 
        ierror = nf90_inquire_dimension(cdfid, timid, len = ntim)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_INQ_VARID(cdfid,dimname(3),timid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_GET_VAR(cdfid,timid,tim(1:ntim))
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
        vardim(4) = ntim
        varmin(4) = tim( 1    )
        varmax(4) = tim( ntim )
        stag  (4) = 0.0     
      endif

c     Read missing data value <misdat>
      ierror  = nf90_get_att(cdfid, varid, "_FillValue", misdat)
      IF(ierror /= nf90_NoErr) THEN
         misdat = -999.99
         ierror = 0
      ENDIF

c     Do some checks
      if ( varmax(1).lt.varmin(1) ) then
         print*,' Error: Wrong order of longitudes'
         ierror = 1
         return
      endif
      if ( varmax(2).lt.varmin(2) ) then
         print*,' Error: Wrong order of latitudes'
         ierror = 1
         return
      endif

         
      end


c     -------------------------------------------------------------------
c     getgrid
c     -------------------------------------------------------------------

      subroutine getgrid_cf(cdfid,dx,dy,ierror)

      use netcdf

      integer   cdfid
      real      dx
      real      dy
      integer   ierror

      integer   lonid
      integer   latid
      real      lat(5000)
      real      lon(5000)
      integer   nlon
      integer   nlat

c     Get the ID for <longitude>, <latitude>, <level> and <time>
      ierror = nf90_inq_dimid(cdfid, 'rlon', lonid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, lonid, len = nlon)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_INQ_VARID(cdfid, 'rlon' ,lonid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inq_dimid(cdfid, 'rlat', latid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, latid, len = nlat)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_INQ_VARID(cdfid, 'rlat' ,latid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Read lon and calculate <dx>
      ierror = NF90_GET_VAR(cdfid,lonid,lon(1:nlon))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      dx = ( lon(nlon) - lon(1) ) / ( real(nlon) - 1.)

c     Read lat and calculate <dy>
      ierror = NF90_GET_VAR(cdfid,latid,lat(1:nlat))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
      dy = ( lat(nlat) - lat(1) ) / ( real(nlat) - 1.)

      end


c     -------------------------------------------------------------------
c     getpole
c     -------------------------------------------------------------------

      subroutine getpole_cf(cdfid,pollon,pollat,polgam,ierror)

      use netcdf

      integer   cdfid, varid
      real      pollon, pollat, polgam
      integer   ierror

      ierror = nf90_inq_varid(cdfid, "rotated_pole", varid)
      ierror  = nf90_get_att(cdfid, varid,
     >                       "grid_north_pole_longitude", pollon)
      if ( ierror.ne.0 ) then
c         print*,' WARNING: rotated pole lon not found -> set to 0'
         pollon = 0.
         ierror = 0
      endif

      ierror  = nf90_get_att(cdfid, varid,
     >                       "grid_north_pole_latitude", pollat)
      if ( ierror.ne.0 ) then
c         print*,' WARNING: rotated pole lat not found -> set to 90'
         pollat = 0.
         ierror = 0
      endif

      ierror  = nf90_get_att(cdfid, varid,
     >                       "north_pole_grid_longitude", polgam)
      if ( ierror.ne.0 ) then
c         print*,' WARNING: rotated pole gamma not found -> set to 0'
         polgam = 0.
         ierror = 0
      endif

      end

c     -------------------------------------------------------------------
c     getcfn
c     -------------------------------------------------------------------

      subroutine getcfn_cf(cdfid,cfn,ierror)
      
      use netcdf

      integer      cdfid
      character*80 cfn
      integer      ierror

      cfn='LMCONSTANTS'

      end


c     -------------------------------------------------------------------
c     getdat
c     -------------------------------------------------------------------

      subroutine getdat_cf(cdfid, varnam, time, level, dat, ierror)

      use netcdf

      integer       cdfid
      character*(*) varnam
      real          time
      integer       level
      real          dat(*)
      integer       ierror

      integer       varid
      integer       ndim
      real          misdat
      integer       vardim(4)
      real          varmin(4)
      real          varmax(4)
      real          stag(4)
      integer       timid
      integer       lonid
      integer       latid
      integer       ntim
      real          tim(5000)
      integer       dimids (nf90_max_var_dims)
      character*80  dimname(nf90_max_var_dims)
      integer       itime
      integer       corner(4)
      integer       count (4)
      real,allocatable, dimension (:,:,:,:) :: tmp
      real,allocatable, dimension (:,:,:)   :: tmp3
      real,allocatable, dimension (:,:)     :: tmp2
      integer       stat
      integer       i,j,k
      character*80  dir

c     Get variable definition
      call getdef_cf (cdfid, varnam, ndim, misdat, 
     &                vardim, varmin, varmax, stag, ierror)

      if ( (ndim.ne.3).and.(ndim.ne.4) ) then
	 print*,' ERROR: getdat only support 3d and 4d arraysd'
         print*,' 3D=time,lat,lon; 4D=time,level,lat,lon'
	 stop
      endif

c     Get <varid>
      ierror = NF90_INQ_VARID(cdfid,varnam,varid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Get <dimids> and <dimname>
      ierror = nf90_inquire_variable(cdfid, varid, 
     >                                   dimids = dimids(1:ndim))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      do i=1,ndim
         ierror = nf90_inquire_dimension(cdfid, dimids(i), 
     >                               name = dimname(i) )
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      enddo

c     Get the ID for <longitude>, <latitude>, <level> and <time>
      if ( ndim.eq.4) then 
         ierror = nf90_inq_dimid(cdfid,dimname(1), lonid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_inq_dimid(cdfid,dimname(2), latid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_inq_dimid(cdfid,dimname(3), levid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_inq_dimid(cdfid,dimname(4), timid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      else if (ndim.eq.3)then
         ierror = nf90_inq_dimid(cdfid,dimname(1), lonid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_inq_dimid(cdfid,dimname(2), latid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_inq_dimid(cdfid,dimname(3), timid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      endif

c     Get all times for variable
      if ( ndim.eq.4 )  then
         ierror = nf90_inquire_dimension(cdfid, timid, len = ntim)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = NF90_INQ_VARID(cdfid,dimname(4),timid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = NF90_GET_VAR(cdfid,timid,tim(1:ntim))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
      elseif ( ndim.eq.3 ) then
         ierror = nf90_inquire_dimension(cdfid, timid, len = ntim)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = NF90_INQ_VARID(cdfid,dimname(3),timid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = NF90_GET_VAR(cdfid,timid,tim(1:ntim))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
      endif
      
c     Find the appropriate time index
      itime = 0
      do i=1,ntim
         if ( tim(i).eq.time) itime = i
      enddo
      if ( itime.eq.0 ) then
         ierror = 1
         print*,'Error: Unknown time in getdat'
         return
      endif

c     Define the data volume to be read
      corner(1) = 1
      count (1) = vardim(1)
      
      corner(2) = 1
      count (2) = vardim(2)
      
      if ( ndim.eq.4 ) then
         if (level.eq.0) then
            corner(3) = 1
            count (3) = vardim(3)
         elseif ( (level.ge.1).and.(level.le.vardim(3)) ) then
            corner(3) = vardim(3)-level+1
            count (3) = 1
         else
            ierror = 1
            return
         endif

         corner(4) = itime
         count (4) = 1

      elseif ( ndim.eq.3 ) then
         
         corner(3) = itime
         count (3) = 1
      
      endif

c     Read data
      if ( ndim.eq.4 ) then
        allocate(tmp(count(1),count(2),count(3),count(4)),stat=stat)
        ierror = nf90_get_var(cdfid,varid,tmp,
     >           start = (/ corner(1),corner(2),corner(3),corner(4) /),
     >           count = (/ count (1),count (2),count (3),count (4) /) ) 

      else if ( ndim.eq.3 ) then
        allocate(tmp3(count(1),count(2),count(3)),stat=stat)
        allocate(tmp(count(1),count(2),1,count(3)),stat=stat)
        ierror = nf90_get_var(cdfid,varid,tmp3,
     >           start = (/ corner(1),corner(2),corner(3) /),
     >           count = (/ count (1),count (2),count (3) /) )
	do j = 1, count(2)
            do i = 1, count(1)
               tmp(i,j,1,1) = tmp3(i,j,1)
            enddo
        enddo
        count(4) = count(3)
        count(3) = 1 
 
      endif

c     De-stagger U/V
      if (trim(dimname(1)) == 'srlon') then
         allocate(tmp2(count(1),count(2)), stat=stat)
         do k = 1, count(3)
            do j = 1, count(2)
               do i = 2, count(1)
                  tmp2(i,j) = 0.5 * (tmp(i,j,k,1) + tmp(i-1,j,k,1))
               enddo
            enddo
            tmp(2:count(1),:,k,1) = tmp2(2:count(1),:)
         enddo
         deallocate(tmp2, stat=stat)
      endif
      if (trim(dimname(2)) == 'srlat') then
         allocate(tmp2(count(1),count(2)), stat=stat)
         do k = 1, count(3)
            do j = 2, count(2)
               do i = 1, count(1)
                  tmp2(i,j) = 0.5 * (tmp(i,j,k,1) + tmp(i,j-1,k,1))
               enddo
            enddo
            tmp(:,2:count(2),k,1) = tmp2(:,2:count(2))
         enddo
         deallocate(tmp2, stat=stat)
      endif

c     Scale specific variables (to be consistent with IVE-varfile)
      if ((trim(varnam) == 'QV') .or. (trim(varnam) == 'QC') .or.
     >    (trim(varnam) == 'QI') .or. (trim(varnam) == 'QR') .or.
     >    (trim(varnam) == 'QS') .or. (trim(varnam) == 'QG') .or.
     >    (trim(varnam) == 'Q_SEDIM') .or.
     >    (trim(varnam) == 'QV_2M'))
     >then
           tmp = tmp * 1000.
           print*,' WARNING: re-scaling ',trim(varnam), ' * 1000'
      endif 
      if ((trim(varnam) == 'P') .or. (trim(varnam) == 'PS') .or.
     >    (trim(varnam) == 'PMSL') .or. (trim(varnam) == 'P_Z'))
     >then
         tmp = tmp * 0.01
         print*,' WARNING: re-scaling ',trim(varnam), ' * 0.01'
      endif
      if ((trim(varnam) == 'T') .or. (trim(varnam) == 'T_2M') .or.
     >    (trim(varnam) == 'TD_2M') .or.
     >    (trim(varnam) == 'TMIN_2M') .or.
     >    (trim(varnam) == 'TMAX_2M') .or.
     >    (trim(varnam) == 'T_P') .or. (trim(varnam) == 'T_Z'))
     >then
         tmp = tmp - 273.15
         print*,' WARNING: re-scaling ',trim(varnam), ' - 273.15'
      endif

c     Copy data to output array: change orientation of vertical axis
      do i=1,count(1)
         do j=1,count(2)
            if ( count(3).gt.1) then

               do k=1,count(3)
                     dat(i+count(1)*(j-1)+count(1)*count(2)*(k-1)) = 
     >                                      tmp(i,j,count(3)-k+1,1)
               enddo

            else 
               dat(i+count(1)*(j-1)) = tmp(i,j,1,1)
            endif

         enddo
      enddo
               
c     Deallocate memory
      deallocate(tmp,stat=stat)

      end

c     -------------------------------------------------------------------
c     gettimes
c     -------------------------------------------------------------------

      subroutine gettimes_cf(cdfid,times,ntimes,ierror)

      use netcdf
      
      integer cdfid
      real    times(*)
      integer ntimes
      integer ierror
      
      integer timid

c     Get all times 
      ierror = nf90_inq_dimid(cdfid,'time', timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, timid, len = ntimes)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_INQ_VARID(cdfid,'time',timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_GET_VAR(cdfid,timid,times(1:ntimes))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end


c     -------------------------------------------------------------------
c     getvars
c     -------------------------------------------------------------------

      subroutine getvars_cf(cdfid,nvars,vnam,ierror)

      use netcdf
      
      integer           cdfid
      integer           nvars
      character*80      vnam(*)
      integer           ierror

      integer           i

c     Get number of variables
      ierror = nf90_inquire(cdfid, nVariables = nvars)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      
c     Get variable names
      do i=1,nvars
          ierror = nf90_inquire_variable(cdfid, i, name = vnam(i) )
          IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      enddo

      return
      end


c***********************************************************************
c CF-netCDF / Writing
c***********************************************************************

c     -------------------------------------------------------------------
c     crecdf
c     -------------------------------------------------------------------

      subroutine crecdf_cf (filnam, cdfid, phymin, phymax, ndim, 
     &                      nlon, nlat, nlev, pollon, pollat, ierror) 
      
      use netcdf
      
      character*(* ) filnam
      integer        cdfid
      real           phymin(4), phymax(4)
      real           pollon, pollat
      real           val(1000)
      integer        ndim, dimid(4), varid(4)
      integer        nlon, nlat, nlev
      integer        ierror, i

c     Create the netCDF file
      ierror = nf90_create(path  = filnam, 
     >                     cmode = nf90_clobber, 
     >                     ncid  = cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Global attributes
      ierror = nf90_put_att(cdfid, nf90_global, 
     >                            "Conventions", "CF")
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, nf90_global, 
     >                             "institution", 'IACETH')
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     longitude dimension
      ierror = nf90_def_dim(cdfid, "rlon", nlon, dimid(1))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_def_var(cdfid, "rlon",
     >                      NF90_FLOAT,dimid(1),varid(1))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(1), 
     >                      "long_name", "rotated longitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(1), 
     >                      "units", "degrees") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(1), 
     >                      "standard_name", "longitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     latitude dimension
      ierror = nf90_def_dim(cdfid, "rlat", nlat, dimid(2))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_def_var(cdfid, "rlat",
     >                      NF90_FLOAT,dimid(2),varid(2))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(2), 
     >                      "long_name", "rotated latitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(2), 
     >                      "units", "degrees") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(2), 
     >                      "standard_name", "latitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     level dimension
      ierror = nf90_def_dim(cdfid, "level", nlev, dimid(3))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_def_var(cdfid, "level",
     >                      NF90_FLOAT,dimid(3),varid(3))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(3), 
     >                      "long_name", "hybrid vertical level") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(3), 
     >                      "units", "level") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     rotated pole
      ierror = nf90_def_var(cdfid, "rotated_pole", NF90_CHAR,
     >                      varid=varid(4))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(4), "long_name",
     >     "coordinates of the rotated north pole")
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(4),
     >     "grid_north_pole_longitude", pollon)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(4),
     >     "grid_north_pole_latitude", pollat)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     End definition mode
      ierror = nf90_enddef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     write longitude, latitude and level
      do i=1,nlon
         val(i) = phymin(1) + real(i-1) *
     >            ( phymax(1) - phymin(1) ) / real(nlon-1)
      enddo
      ierror = nf90_put_var(cdfid, varid(1), val(1:nlon))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      do i=1,nlat
         val(i) = phymin(2) + real(i-1) *
     >            ( phymax(2) - phymin(2) ) / real(nlat-1)
      enddo
      ierror = nf90_put_var(cdfid, varid(2), val(1:nlat))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      do i=1,nlev
         val(i) = real(i) - 1.
      enddo
      ierror = nf90_put_var(cdfid, varid(3), val(1:nlev))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      end


c     -------------------------------------------------------------------
c     putdef
c     -------------------------------------------------------------------

      subroutine putdef_cf (cdfid, newvar, ndimension, misdat, 
     &                      vardim, varmin, varmax, stag, ierror)

      use netcdf
      
      integer        cdfid
      character*(*)  newvar
      integer        ndimension
      real           misdat
      integer        vardim(4)
      real           varmin(4)
      real           varmax(4)
      real           stag(4)
      integer        ierror

      real          eps
      parameter     (eps=0.0001)
      
      integer       ndim
      integer       nvar
      character*80  dimname(100)
      character*80  varname(100)
      integer       dimid
      integer       dimids(4)
      integer       nval
      real          val(1000)
      integer       varid
      integer       varids(4)
      integer       writelon,writelat,writelev,writetim
      character*80  lonname,latname,levname,timname

c     Unit/name table
      character*80  tablefile
      parameter     (tablefile='nil')
      character*80  table_varname (1000)
      character*80  table_longname(1000)
      character*80  table_unit    (1000)
      integer       fid
      integer       ntable

c     Read table information
      if ( tablefile.ne.'nil' ) then
         ntable = 0
         fid    = 10
         open(fid,file=tablefile)
 100      ntable = ntable + 1
          read(fid,*) table_varname (ntable),
     >                        table_longname(ntable),
     >                        table_unit    (ntable)

          print*,table_varname(ntable),table_longname(ntable),
     >             table_unit(ntable)
          
          goto 100
 101     close(fid)
      endif

c     Set the default names for dimensions
      lonname = 'rlon'
      latname = 'rlat'
      levname = 'level'
      timname = 'time'

c     Get number of dimensions and of variables
      ierror = nf90_inquire(cdfid, nDimensions    = ndim,
     >                             nVariables     = nvar )
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Get a list of all dimensions
      if ( ndim.gt.0 ) then
         do i=1,ndim
            ierror = nf90_inquire_dimension(cdfid,i,name = dimname(i))
            IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         enddo
      endif

c     Get a list of all variables
      if ( nvar.gt.0 ) then
         do i=1,nvar
            ierror = nf90_inquire_variable(cdfid,i,name = varname(i))
            IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         enddo
      endif

c     Check whether the new variable name is in conflict with the others
      do i=1,nvar
         if ( newvar.eq.varname(i) ) then
            print*,trim(newvar),' already exists on the file'
            stop
         endif
      enddo

c     Set IDs for dimensions 
      do i=1,4
         dimids(i) = -1
      enddo
      writelon = 0
      writelat = 0
      writelev = 0
      writetim = 0

c     Check whether existing dimensions can be used
      do i=1,ndim

c        Read dimension values
         ierror = nf90_inquire_dimension(cdfid,i,len = nval)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)         
         ierror = NF90_INQ_VARID(cdfid,dimname(i),varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
         ierror = NF90_GET_VAR(cdfid,varid,val(1:nval))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

c        Check whether it can be used for the first dimension (lon)
         if ( (vardim(1).eq.nval).and.
     >        (abs(varmin(1)-val(1)   ).lt.eps).and.
     >        (abs(varmax(1)-val(nval)).lt.eps) )
     >   then
            dimids(1) = i
         endif

c        Check whether it can be used for the second dimension (lat)
         if ( (vardim(2).eq.nval).and.
     >        (abs(varmin(2)-val(1)   ).lt.eps).and.
     >        (abs(varmax(2)-val(nval)).lt.eps) )
     >   then
            dimids(2) = i
         endif
         
c        Check whether it can be used for the third dimension (lev)
         if ( (vardim(3).eq.nval) ) then
            dimids(3) = i
         endif
         
c        Check whether there is already a time dimension (tim)
         ierror = nf90_inquire(cdfid, unlimitedDimId = dimids(4) )
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      enddo

c     Decide what dimension names to use
      if ( dimids(1).eq.-1 ) then
         do i=1,ndim
            if ( dimname(i).eq.lonname ) then
               lonname = trim(lonname)//'_'//trim(newvar)
            endif
         enddo
      endif
      if ( dimids(2).eq.-1 ) then
         do i=1,ndim
            if ( dimname(i).eq.latname ) then
               latname = trim(latname)//'_'//trim(newvar)
            endif
         enddo
      endif
      if ( dimids(3).eq.-1 ) then
         do i=1,ndim
            if ( dimname(i).eq.levname ) then
               levname = trim(levname)//'_'//trim(newvar)
            endif
         enddo
      endif

c     Enter definition mode
      ierror = nf90_redef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Define dimensions and corresponding variables if necessary
      if ( dimids(1).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid, lonname, vardim(1), dimids(1))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, lonname,
     >                                 NF90_FLOAT,dimids(1),varids(1))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(1), 
     >        "long_name", "rotated longitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(1), 
     >        "units", "degrees") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(1), 
     >        "standard_name", "longitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         writelon = 1
      endif
      if ( dimids(2).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid, latname, vardim(2), dimids(2))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, latname,
     >                                NF90_FLOAT,dimids(2),varids(2))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(2), 
     >        "long_name", "rotated latitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(2), 
     >        "units", "degrees") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(2), 
     >        "standard_name", "latitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         writelat = 1
      endif
      if ( dimids(3).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid, levname, vardim(3), dimids(3))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, levname,
     >                                NF90_FLOAT,dimids(3),varids(3))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(3), 
     >        "long_name", "hybrid vertical level")
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(3), 
     >        "units", "level") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         writelev = 1
      endif
      if ( dimids(4).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid,timname, nf90_unlimited, dimids(4))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, timname,
     >                                NF90_FLOAT,dimids(4),varids(4))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(4), 
     >                 "units", "hours") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(4), 
     >                 "calendar", "proleptic_gregorian") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         
         writetim = 1
      endif

c     Define the variable
      ierror = nf90_def_var(cdfid,newvar,
     >                      NF90_FLOAT,dimids,varid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Write variable attributes
      ierror = nf90_put_att(cdfid, varid, 
     >                 "long_name", "unknown (please add with NCO)") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid, 
     >                 "units", "unknown (please add with NCO)") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid, 
     >                 "_FillValue", real(-999.99) ) 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)


c     Leave definition mode
      ierror = nf90_enddef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Write dimension arrays
      if ( writelon.eq.1 ) then
         do i=1,vardim(1)
            val(i) = varmin(1) + real(i-1) *
     >                 ( varmax(1) - varmin(1) ) / real(vardim(1)-1)
         enddo
         ierror = nf90_inq_varid(cdfid,lonname,varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_var(cdfid, varid, val(1:vardim(1)))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      endif
      if ( writelat.eq.1 ) then
         do i=1,vardim(2)
            val(i) = varmin(2) + real(i-1) *
     >                 ( varmax(2) - varmin(2) ) / real(vardim(2)-1)
         enddo
         ierror = nf90_inq_varid(cdfid,latname,varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_var(cdfid, varid, val(1:vardim(2)))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      endif

      if ( writelev.eq.1 ) then
         do i=1,vardim(3)
            val(i) = real(i)
         enddo
         ierror = nf90_inq_varid(cdfid,levname,varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_var(cdfid, varid, val(1:vardim(3)) )
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      endif

      end


c     -------------------------------------------------------------------
c     puttimes
c     -------------------------------------------------------------------

      subroutine puttimes_cf(cdfid,times,ntimes,ierror)

      use netcdf
      
      integer   cdfid
      real      times(*)
      integer   ntimes
      integer   ierror

      integer   timid
      integer   otimes

c     Get number of times 
      ierror = nf90_inq_dimid(cdfid,'time', timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, timid, len = otimes)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      
      if (otimes.lt.ntimes) then
         print *,'Warning: puttimes is extending times-array'
      else if (nfiltim.gt.ntimes) then
         print *,'Warning: puttimes does not cover range of times-array'
      endif

c     Write times
      ierror = NF90_INQ_VARID(cdfid,'time',timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_PUT_VAR(cdfid,timid,times(1:ntimes))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

c     Sync
      ierror = nf90_sync(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end


c     -------------------------------------------------------------------
c     putstart
c     -------------------------------------------------------------------

      subroutine putstart_cf(cdfid,idate,ierror)

      use netcdf

      integer      cdfid
      integer      idate(5)
      integer      ierror

c     Enter definition mode
      ierror = nf90_redef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Write data      
      ierror  = nf90_put_att(cdfid,nf90_global, "starty", idate(1))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "startm", idate(2))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "startd", idate(3))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "starth", idate(4))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "starts", idate(5))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Exit definition mode
      ierror = nf90_enddef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Sync
      ierror = nf90_sync(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end


c     -------------------------------------------------------------------
c     cdfwopn
c     -------------------------------------------------------------------

      subroutine cdfwopn_cf(filnam,cdfid,ierror)
      
      use netcdf
      
      character*(*) filnam
      integer       cdfid
      integer       ierror

      ierror = NF90_OPEN(TRIM(filnam),nf90_write, cdfid)
      IF ( ierror /= nf90_NoErr ) PRINT *,NF90_STRERROR(ierror)

      return
      end


c     -------------------------------------------------------------------
c     putdat
c     -------------------------------------------------------------------

      subroutine putdat_cf(cdfid, varnam, time, level, dat, ierror)

      use netcdf
      
      integer        cdfid
      character*(*)  varnam
      real           time
      integer        level
      real           dat(*)
      integer        ierror

      real           eps
      parameter      (eps=0.0001)

      integer        ndims
      real           misdat
      integer        vardim(4)
      real           varmin(4)
      real           varmax(4)
      real           stag(4)
      integer        varid
      integer        ntimes
      real           times(100)
      integer        itime
      integer        corner(4)
      integer        count(4)
      integer        stat
      real,allocatable, dimension (:,:,:,:) :: tmp

c     Get definitions of data
      call getdef_cf (cdfid, trim(varnam), ndims, misdat, 
     &                vardim, varmin, varmax, stag, ierr)
      if (ierr.ne.0)  print *,'*ERROR* in getdef in putdat'

c     Get id of the variable
      ierror = NF90_INQ_VARID(cdfid,varnam,varid)
      IF (ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Get a list of all times 
      call gettimes_cf(cdfid,times,ntimes,ierror)

c     Get the time index
      itime = -1
      do i=1,ntimes
         if ( abs(time-times(i)).lt.eps) itime = i
      enddo

c     Start a new time step, if necessary
      if ( itime.eq.-1 ) then
         ntimes        = ntimes + 1
         times(ntimes) = time
         itime         = ntimes
         call puttimes_cf(cdfid,times,ntimes,ierror)
      endif

C     Define data volume to write on the NetCDF file in index space
      corner(1) = 1               
      corner(2) = 1
      count(1)  = vardim(1)       
      count(2)  = vardim(2)
      if (level.eq.0) then
        corner(3) = 1
        count(3)  = vardim(3)
      else
        corner(3) = vardim(3)-level+1
        count(3)  = 1
      endif
      corner(4) = itime
      count(4)  = 1

c     Allocate temporary array
      allocate(tmp(count(1),count(2),count(3),count(4)),stat=stat)

c     Copy data to output array: change orientation of vertical axis
      if ( count(3).gt.1 ) then
         do i=1,count(1)
            do j=1,count(2)
               do k=1,count(3)
                  tmp(i,j,count(3)-k+1,1) =
     >               dat(i+count(1)*(j-1)+count(1)*count(2)*(k-1)) 
               enddo
            enddo
         enddo
      else
         do i=1,count(1)
            do j=1,count(2)
               tmp(i,j,1,1) = dat(i+count(1)*(j-1)) 
            enddo
         enddo
      endif

c    Re-scale specific variables
      if ((trim(varnam) == 'QV') .or. (trim(varnam) == 'QC') .or.
     >    (trim(varnam) == 'QI') .or. (trim(varnam) == 'QR') .or.
     >    (trim(varnam) == 'QS') .or. (trim(varnam) == 'QG') .or.
     >    (trim(varnam) == 'Q_SEDIM') .or.
     >    (trim(varnam) == 'QV_2M')) 
     >then
           tmp = tmp / 1000.
           print*,' WARNING: re-scaling ',trim(varnam),' 1/1000'
      endif 
      if ((trim(varnam) == 'P') .or. (trim(varnam) == 'PS') .or.
     >    (trim(varnam) == 'PMSL') .or. (trim(varnam) == 'P_Z'))
     >then
           tmp = tmp / 0.01
           print*,' WARNING: re-scaling ',trim(varnam),' 1/0.01'
      endif
      if ((trim(varnam) == 'T') .or. (trim(varnam) == 'T_2M') .or.
     >    (trim(varnam) == 'TD_2M') .or.
     >    (trim(varnam) == 'TMIN_2M') .or.
     >    (trim(varnam) == 'TMAX_2M') .or.
     >    (trim(varnam) == 'T_P') .or. (trim(varnam) == 'T_Z'))
     >then
         tmp = tmp + 273.15
         print*,' WARNING: re-scaling ',trim(varnam),' + 273.15'
      endif

c     Write data
      ierror = nf90_put_var(cdfid, varid, tmp,
     >           start = (/ corner(1),corner(2),corner(3),corner(4) /),
     >           count = (/ count (1),count (2),count (3),count (4) /) ) 

c     Deallocate memory
      deallocate(tmp,stat=stat)

c     Sync
      ierror = nf90_sync(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end

 




 
c***********************************************************************
c IVE-netCDF
c***********************************************************************


      subroutine clscdf_ive (cdfid, error)
c-----------------------------------------------------------------------
c     Purpose:
c        This routine closes an open netCDF file.
c     Aguments :
c        cdfid  int  input   the id of the file to be closed.
c        error  int  output  indicates possible errors found in this
c                            routine.
c                            error = 0   no errors detected.
c                            error = 1   error detected.
c     History:
c        Nov. 91  PPM  UW  Created.
c-----------------------------------------------------------------------

      include "netcdf.inc"

c     Argument declarations.
      integer      cdfid, error

c     Local variable declarations.
      integer      ncopts

c     Get current value of error options.
      call ncgopt (ncopts)

c     Make sure netCDF errors do not abort execution.
      call ncpopt (NCVERBOS)

c     Close requested file.
      call ncclos (cdfid, error)

c     Reset error options.
      call ncpopt (ncopts)

      end


      subroutine crecdf_ive (filnam, cdfid, phymin, phymax, ndim, 
     &                       nlon, nlat, nlev, pollon, pollat, error) 
c-----------------------------------------------------------------------
c     Purpose:
c        This routine is called to create a netCDF file for use with 
c        the UWGAP plotting package.
c           Any netCDF file written to must be closed with the call
c        'call clscdf(cdfid,error)', where cdfid and error are
c        as in the argumentlist below. 
c     Arguments:
c        filnam  char  input   the user-supplied netCDF file name.
c        cdfid   int   output  the file-identifier
c        phymin  real  input   the minimum physical dimension of the
c                              entire physical domain along each axis.
c                              phymin is dimensioned (ndim)
c        phymax  real  input   the maximum physical dimension of the
c                              entire physical domain along each axis.
c                              phymax is dimensioned (ndim)
c        ndim    int   input   the number of dimensions in the file
c                              (i.e. number of elements in phymin,
c                              phymax)
c        error   int   output  indicates possible errors found in this
c                              routine.
c                              error = 0   no errors detected.
c                              error = 1   error detected.
c     History:
c        Nov. 91  PPM  UW  Created cr3df.
c        Jan. 92  CS   UW  Created crecdf.
c-----------------------------------------------------------------------

      include "netcdf.inc"

c     Argument declarations.
      integer        MAXDIM
      parameter      (MAXDIM=4)
      integer        ndim, error, nlon, nlat, nlev
      character *(*) filnam
      real           phymin(*), phymax(*)
      real           pollon, pollat

c     Local variable declarations.
      character *(20) attnam
      character *(1)  chrid(MAXDIM)
      integer         cdfid, k, ibeg, iend, lenfil, ncopts
      data            chrid/'x','y','z','a'/

c     Get current value of error options, and make sure netCDF-errors do
c     not abort execution
      call ncgopt (ncopts)
      call ncpopt(NCVERBOS)

c     Initially set error to indicate no errors.
      error = 0

c     create the netCDF file
      cdfid = nccre (trim(filnam), NCCLOB, error)
      if (error.ne.0) go to 920

c     define global attributes
      do k=1,ndim
        attnam(1:3)='dom'
        attnam(4:4)=chrid(k)
        attnam(5:7)='min'
        attnam=attnam(1:7)
        call ncapt(cdfid,NCGLOBAL,attnam,NCFLOAT,1,phymin(k),error)
        if (error.gt.0) goto 920

        attnam(1:3)='dom'
        attnam(4:4)=chrid(k)
        attnam(5:7)='max'
        attnam=attnam(1:7)
        call ncapt(cdfid,NCGLOBAL,attnam,NCFLOAT,1,phymax(k),error)
        if (error.gt.0) goto 920
      enddo
      call ncapt(cdfid,NCGLOBAL,'nx',NCSHORT,1,nlon,error)
      if (error.gt.0) goto 920
      call ncapt(cdfid,NCGLOBAL,'ny',NCSHORT,1,nlat,error)
      if (error.gt.0) goto 920
      call ncapt(cdfid,NCGLOBAL,'nz',NCSHORT,1,nlev,error)
      if (error.gt.0) goto 920

c     define pole coordinates
      call ncapt(cdfid,NCGLOBAL,'pollon',NCFLOAT,1,pollon,error)
      if (error.gt.0) goto 920
      call ncapt(cdfid,NCGLOBAL,'pollat',NCFLOAT,1,pollat,error)
      if (error.gt.0) goto 920

c     Define conventions
      call ncaptc (cdfid, NCGLOBAL, 'Conventions',NCCHAR,3,'IVE',error)

c     End variable definitions.
      call ncendf (cdfid, error)
      if (error.gt.0) goto 920

c     normal exit
      call ncpopt (ncopts)
      return

c     error exit
 920  write (6, *) 'ERROR: An error occurred while attempting to ',
     &             'create the data file in subroutine crecdf.'
      call ncpopt (ncopts)
      call ncclos (cdfid, error)
      end


      subroutine getdef_ive (cdfid, varnam, ndim, misdat, 
     &                              vardim, varmin, varmax, stag, error)
c-----------------------------------------------------------------------
c     Purpose:
c        This routine is called to get the dimensions and attributes of 
c        a variable from an IVE-NetCDF file for use with the IVE plotting
c        package. Prior to calling this routine, the file must be opened
c        with a call to opncdf.
c     Arguments:
c        cdfid   int   input   file-identifier
c                              (can be obtained by calling routine 
c                              opncdf)
c        varnam  char  input   the user-supplied variable name.
c                              (can be obtained by calling routine 
c                              opncdf)
c        ndim    int   output  the number of dimensions (ndim<=4)
c        misdat  real  output  missing data value for the variable. 
c        vardim  int   output  the dimensions of the variable.
c                              Is dimensioned at least (ndim). 
c        varmin  real  output  the location in physical space of the
c                              origin of each variable. 
c                              Is dimensioned at least Min(3,ndim). 
c        varmax  real  output  the extend of each variable in physical
c                              space.
c                              Is dimensioned at least Min(3,ndim). 
c        stag    real  output  the grid staggering for each variable.
c                              Is dimensioned at least Min(3,ndim). 
c        error   int   output  indicates possible errors found in this
c                              routine.
c                              error = 0   no errors detected.
c                              error = 1   the variable is not on the file.
c                              error =10   other errors.
c     History:
c       Apr. 93    Christoph Schaer (ETHZ)     Created.
c-----------------------------------------------------------------------

      include "netcdf.inc"

c     Argument declarations.
      integer        MAXDIM
      parameter      (MAXDIM=4)
      character *(*) varnam
      integer        vardim(*), ndim, error, cdfid
      real           misdat,  stag(*), varmin(*), varmax(*)

c     Local variable declarations.
      character *(20) dimnam(MAXNCDIM),attnam,vnam
      character *(1)  chrid(MAXDIM)
      integer         id,i,k
      integer         ndims,nvars,ngatts,recdim,dimsiz(MAXNCDIM)
      integer         vartyp,nvatts, ncopts
      data            chrid/'x','y','z','t'/

c     Get current value of error options.
      call ncgopt (ncopts)

c     make sure NetCDF-errors do not abort execution
      call ncpopt(NCVERBOS)

c     Initially set error to indicate no errors.
      error = 0

c     inquire for number of dimensions
      call ncinq(cdfid,ndims,nvars,ngatts,recdim,error)
      if (error.eq.1) goto 920

c     read dimension-table
      do i=1,ndims 
        call ncdinq(cdfid,i,dimnam(i),dimsiz(i),error)
        if (error.gt.0) goto 920
      enddo

c     get id of the variable
      id=ncvid(cdfid,varnam,error)
      if (error.eq.1) goto 910

c     inquire about variable
      call ncvinq(cdfid,id,vnam,vartyp,ndim,vardim,nvatts,error)
      if (vartyp.ne.NCFLOAT) error=1
      if (error.gt.0) goto 920

c     Make sure ndim <= MAXDIM.
      if (ndim.gt.MAXDIM) then
         error = 1
         go to 900
      endif

c     get dimensions from dimension-table
      do k=1,ndim 
        vardim(k)=dimsiz(vardim(k))
      enddo

c     get attributes
      do k=1,min0(3,ndim)
c       get min postion
        attnam(1:1)=chrid(k)
        attnam(2:4)='min'
        attnam=attnam(1:4)
        call ncagt(cdfid,id,attnam,varmin(k),error)
        if (error.gt.0) goto 920
c       get max position     
        attnam(1:1)=chrid(k)
        attnam(2:4)='max'
        attnam=attnam(1:4)
        call ncagt(cdfid,id,attnam,varmax(k),error)
        if (error.gt.0) goto 920     
c       get staggering
        attnam(1:1)=chrid(k)
        attnam(2:5)='stag'
        attnam=attnam(1:5)
        call ncagt(cdfid,id,attnam,stag(k),error)
        if (error.gt.0) goto 920
      enddo

c     get missing data value
      call ncagt(cdfid,id,'missing_data',misdat,error)
      if (error.gt.0) goto 920     

c     normal exit
      call ncpopt (ncopts)
      return

c     Error exits.
 900  write (6, *) '*ERROR*: When calling getcdf, the number of ',
     &             'variable dimensions must be less or equal 4.'
      call ncpopt (ncopts)
      call ncclos (cdfid, error)
      return

 910  write (6, *) '*ERROR*: The selected variable could not be found ',       
     &             'in the file by getcdf.'
      call ncpopt (ncopts)
      call ncclos (cdfid, error)
      return

 920  write (6, *) '*ERROR*: An error occurred while attempting to ',
     &             'read the data file in subroutine getcdf.'
      call ncpopt (ncopts)
      call ncclos (cdfid, error)
      end



      subroutine getdat_ive(cdfid, varnam, time, level, dat, error)
c-----------------------------------------------------------------------
c     Purpose:
c        This routine is called to read the data of a variable
c        from an IVE-NetCDF file for use with the IVE plotting package. 
c        Prior to calling this routine, the file must be opened with 
c        a call to opncdf (for extension) or crecdf (for creation) or
c        readcdf (for readonly).
c     Arguments:
c        cdfid   int   input   file-identifier
c                              (must be obtained by calling routine 
c                              opncdf,readcdf  or crecdf)
c        varnam  char  input   the user-supplied variable name (must 
c                              previously be defined with a call to
c                              putdef)
c        time    real  input   the user-supplied time-level of the
c                              data to be read from the file (the time-
c                              levels stored in the file can be obtained
c                              with a call to gettimes). 
c        level   int   input   the horizontal level(s) to be read 
c                              to the NetCDF file. Suppose that the
c                              variable is defined as (nx,ny,nz,nt).
c                              level>0: the call reads the subdomain
c                                       (1:nx,1:ny,level,itimes)
c                              level=0: the call reads the subdomain
c                                       (1:nx,1:ny,1:nz,itimes)
c                              Here itimes is the time-index corresponding
c                              to the value of 'time'. 
c        dat     real  output  data-array dimensioned sufficiently 
c                              large. The dimensions (nx,ny,nz)
c                              of the variable must previously be defined
c                              with a call to putdef. No previous 
c                              definition of the time-dimension is
c                              required.
c        error   int   output  indicates possible errors found in this
c                              routine.
c                              error = 0   no errors detected.
c                              error = 1   the variable is not present on
c                                          the file.
c                              error = 2   the value of 'time' is not
c                                          known.to the file.
c                              error = 3   inconsistent value of level
c                              error =10   another error.
c     History:
c       March 93    Heini Wernli (ETHZ)      Created wr2cdf.
c       April 93    Bettina Messmer (ETHZ)   Created putdat.
c       June  93    Christoph Schaer (ETHZ)  Created getdat
c       Sept. 07    Johannes Jenkner (ETHZ)  Integer and double times
c-----------------------------------------------------------------------

      include "netcdf.inc"

C     Declaration of local variables
      character*(*) varnam
      character*(20) chars,dimnam
      integer cdfid

      real        dat(*)
      real        misdat,varmin(4),varmax(4),stag(4)
      real        time, timeval

      integer     corner(4),edgeln(4),didtim,vardim(4),ndims
      integer     error, ierr
      integer     level,ntime
      integer     idtime,idvar,iflag
      integer     i

      integer     vtyp,dn,nat
      integer     dims(4)
      integer     inttime
      double precision doubletime

      call ncpopt(NCVERBOS)

c     access the variable
      call getdef (cdfid, trim(varnam), ndims, misdat, 
     &                           vardim, varmin, varmax, stag, ierr)
      if (ierr.ne.0) then
        print *,'*ERROR* in getdef in getdat'
        error=1
        return
      endif
      idvar=ncvid(cdfid,trim(varnam),ierr)
      if (ierr.ne.0) then
        print *,'*ERROR* in ncvid in getdat'
        error=1
        return
      endif

C     Get times-array
      didtim=ncdid(cdfid,'time',ierr)
      if (ierr.ne.0) then
        print *,'*ERROR* didtim in getdat'
        error=10
        return
      endif
      call ncdinq(cdfid,didtim,chars,ntime,ierr)
      if (ierr.ne.0) then
        print *,'*ERROR* in ncdinq in getdat'
        error=10
        return
      endif
      idtime=ncvid(cdfid,'time',ierr)
      if (ierr.ne.0) then
        print *,'*ERROR* in ncvid for time in getdat'
        error=10
        return
      endif
c     find appropriate time-index
      iflag=0
      call ncvinq(cdfid,idtime,dimnam,vtyp,dn,dims,natt,ierr)
      do i=1,ntime
        if (vtyp.eq.5) then
          call ncvgt1(cdfid,idtime,i,timeval,ierr)
        elseif (vtyp.eq.4) then ! integer version 
          call ncvgt1(cdfid,idtime,i,inttime,ierr) 
          timeval=real(inttime)
        elseif (vtyp.eq.6) then ! double precision version
           call ncvgt1(cdfid,idtime,i,doubletime,ierr) 
           timeval=real(doubletime)   
        endif      
        if (ierr.ne.0) print *,'*ERROR* in ncvgt1 in getdat'
        if (time.eq.timeval) iflag=i
      enddo
      if (iflag.eq.0) then
        error=2
        print *,'Error: Unknown time in getdat'
        return
      endif

C     Define data volume to be written (index space)
      corner(1)=1
      corner(2)=1
      edgeln(1)=vardim(1)
      edgeln(2)=vardim(2)
      if (level.eq.0) then
        corner(3)=1
        edgeln(3)=vardim(3)
      else if ((level.le.vardim(3)).and.(level.ge.1)) then
        corner(3)=level
        edgeln(3)=1
      else
        error=3
        return
      endif
      corner(4)=iflag
      edgeln(4)=1

C     Read data from NetCDF file
c      print *,'getdat vor Aufruf ncvgt'
c      print *,'cdfid ',cdfid
c      print *,'idvar ',idvar
c      print *,'corner ',corner
c      print *,'edgeln ',edgeln

      call ncvgt(cdfid,idvar,corner,edgeln,dat,error)
      if (error.ne.0) then
        print *, '*ERROR* in ncvgt in getdat'
        error=10
      endif

      end


      subroutine putdat_ive(cdfid, varnam, time, level, dat, error)
c-----------------------------------------------------------------------
c     Purpose:
c        This routine is called to write the data of a variable
c        to an IVE-NetCDF file for use with the IVE plotting package. 
c        Prior to calling this routine, the file must be opened with 
c        a call to opncdf (for extension) or crecdf (for creation), the 
c        variable must be defined with a call to putdef.
c     Arguments:
c        cdfid   int   input   file-identifier
c                              (must be obtained by calling routine 
c                              opncdf or crecdf)
c        varnam  char  input   the user-supplied variable name (must 
c                              previously be defined with a call to
c                              putdef)
c        time    real  input   the user-supplied time-level of the
c                              data to be written to the file (the time-
c                              levels stored in the file can be obtained
c                              with a call to gettimes). If 'time' is not
c                              yet known to the file, a knew time-level is
c                              allocated and appended to the times-array.
c        level   int input     the horizontal level(s) to be written 
c                              to the NetCDF file. Suppose that the
c                              variable is defined as (nx,ny,nz,nt).
c                              level>0: the call writes the subdomain
c                                       (1:nx,1:ny,level,itimes)
c                              level=0: the call writes the subdomain
c                                       (1:nx,1:ny,1:nz,itimes)
c                              Here itimes is the time-index corresponding
c                              to the value of 'time'. 
c        dat     real  output  data-array dimensioned sufficiently 
c                              large. The dimensions (nx,ny,nz)
c                              of the variable must previously be defined
c                              with a call to putdef. No previous 
c                              definition of the time-dimension is
c                              required.
c        error   int output    indicates possible errors found in this
c                              routine.
c                              error = 0   no errors detected.
c                              error = 1   the variable is not present on
c                                          the file.
c                              error = 2   the value of 'time' is new, but
c                                          appending it would yield a non
c                                          ascending times-array.
c                              error = 3   inconsistent value of level
c                              error =10   another error.
c     History:
c       March 93    Heini Wernli (ETHZ)      Created wr2cdf.
c       April 93    Bettina Messmer (ETHZ)   Created putdat.
c-----------------------------------------------------------------------

      include "netcdf.inc"

C     Declaration of local variables

      character*(*) varnam
      character*(20) chars
      integer cdfid


      real 	dat(*)
      real	misdat,varmin(3),varmax(3),stag(3)
      real	time, timeval
      data	stag/0.,0.,0./

      integer	corner(4),edgeln(4),did(4),vardim(4),ndims
      integer	error, ierr
      integer	level,ntime
      integer	idtime,idvar,iflag
      integer	i

      call ncpopt(NCVERBOS)

c     get definitions of data
      call getdef (cdfid, trim(varnam), ndims, misdat, 
     &                           vardim, varmin, varmax, stag, ierr)
      if (ierr.ne.0)  print *,'*ERROR* in getdef in putdat'

c     get id of variable
      idvar=ncvid(cdfid,trim(varnam),ierr)
      if (ierr.ne.0) print *,'*ERROR* in ncvid in putdat'

c     get times-array 
      did(4)=ncdid(cdfid,'time',ierr)
      if (ierr.ne.0) print *,'*ERROR* did(4) in putdat'
      call ncdinq(cdfid,did(4),chars,ntime,ierr)
      if (ierr.ne.0) print *,'*ERROR* in ncdinq in putdat'
      idtime=ncvid(cdfid,'time',ierr)
      if (ierr.ne.0) print *,'*ERROR* in ncvid in putdat'
C     Check if a new time step is starting
      iflag=0
      do i=1,ntime
        call ncvgt1(cdfid,idtime,i,timeval,ierr)
        if (ierr.ne.0) print *,'*ERROR* in ncvgt1 in putdat'
        if (time.eq.timeval) iflag=i
      enddo
      if (iflag.eq.0) then		! new time step
        ntime=ntime+1
        iflag=ntime
        idtime=ncvid(cdfid,'time',ierr)
        if (ierr.ne.0) print *, '*ERROR* in ncvid in putdat'
        call ncvpt1(cdfid,idtime,ntime,time,ierr)
        if (ierr.ne.0) print *, '*ERROR* in ncvpt1 in putdat'
      endif

C     Define data volume to write on the NetCDF file in index space
      corner(1)=1               ! starting corner of data volume
      corner(2)=1
      edgeln(1)=vardim(1)       ! edge lengthes of data volume
      edgeln(2)=vardim(2)
      if (level.eq.0) then
        corner(3)=1
        edgeln(3)=vardim(3)
      else
        corner(3)=level
        edgeln(3)=1
      endif
      corner(4)=iflag
      edgeln(4)=1
      
C     Put data on NetCDF file

c      print *,'vor Aufruf ncvpt d.h. Daten schreiben in putdat '
c      print *,'cdfid ',cdfid
c      print *,'idvar ',idvar
c      print *,'corner ',corner
c      print *,'edgeln ',edgeln

      call ncvpt(cdfid,idvar,corner,edgeln,dat,error)
      if (error.ne.0) then
        print *, '*ERROR* in ncvpt in putdat - Put data on NetCDF file'
      endif

C     Synchronize output to disk and close the files

      call ncsnc(cdfid,ierr)
      if (ierr.ne.0) print *, '*ERROR* in ncsnc in putdat'
      end






      subroutine putdef_ive (cdfid, varnam, ndim, misdat, 
     &                            vardim, varmin, varmax, stag, error)
c-----------------------------------------------------------------------
c     Purpose:
c        This routine is called to define the dimensions and the
c        attributes of a variable on an IVE-NetCDF file for use with the
c        IVE plotting package. Prior to calling this routine, the file must
c        be opened with a call to opncdf (extend an existing file) or
c        crecdf (create a new file).
c     Arguments:
c        cdfid   int   input   file-identifier
c                              (can be obtained by calling routine 
c                              opncdf)
c        varnam  char  input   the user-supplied variable name.
c        ndim    int   input   the number of dimensions (ndim<=4). 
c                              Upon ndim=4, the fourth dimension of the
c                              variable is specified as 'unlimited'
c                              on the file (time-dimension). It can 
c                              later be extended to arbitrary length.
c        misdat  real  input   missing data value for the variable. 
c        vardim  int   input   the dimensions of the variable.
c                              Is dimensioned at least Min(3,ndim). 
c        varmin  real  input   the location in physical space of the
c                              origin of each variable.
c                              Is dimensioned at least Min(3,ndim). 
c        varmax  real  input   the extent of each variable in physical
c                              space.
c                              Is dimensioned at least Min(ndim). 
c        stag    real  input   the grid staggering for each variable.
c                              Is dimensioned at least Min(3,ndim). 
c        error   int   output  indicates possible errors found in this
c                              routine.
c                              error = 0   no errors detected.
c                              error =10   other errors detected.
c     History:
c       Apr. 93    Christoph Schaer (ETHZ)     Created.
c-----------------------------------------------------------------------

      include "netcdf.inc"

c     Argument declarations.
      integer        MAXDIM
      parameter      (MAXDIM=4)
      character *(*) varnam
      integer        vardim(*), ndim, error, cdfid
      real           misdat,  stag(*), varmin(*), varmax(*)

c     Local variable declarations.
      character *(20) dimnam,attnam,dimchk
      character *(1)  chrid(MAXDIM)
      character *(20) dimnams(MAXNCDIM)
      integer         dimvals(MAXNCDIM)
      integer         numdims,numvars,numgats,dimulim
      integer         id,did(MAXDIM),idtime,i,k,ierr
      integer         ncopts
      integer         ibeg,iend
      data            chrid/'x','y','z','t'/

c     Get current value of error options.
      call ncgopt (ncopts)

c     make sure NetCDF-errors do not abort execution
      call ncpopt(NCVERBOS)

c     Initially set error to indicate no errors.
      error = 0

c     Make sure ndim <= MAXDIM.
      if (ndim.gt.MAXDIM) then
         error = 10
         go to 900
      endif

c     Read existing dimensions-declarations from the file
      call ncinq(cdfid,numdims,numvars,numgats,dimulim,error)
      if (numdims.gt.0) then
        do i=1,numdims
          call ncdinq(cdfid,i,dimnams(i),dimvals(i),error)
c         print *,dimnams(i),dimvals(i)
        enddo
      endif

c     put file into define mode
      call ncredf(cdfid,error)
      if (error.ne.0) goto 920

c     define spatial dimensions
      do k=1,min0(3,ndim)
c       define the default dimension-name
        dimnam(1:3)='dim'
        dimnam(4:4)=chrid(k)
        dimnam(5:5)='_'
        dimnam(6:5+len_trim(varnam))=trim(varnam)
        dimnam=dimnam(1:5+len_trim(varnam))
        did(k)=-1
        if (numdims.gt.0) then
c         check if an existing dimension-declaration can be used
c         instead of defining a new dimension
          do i=1,numdims
            dimchk=dimnams(i)
            if ((vardim(k).eq.dimvals(i)).and.
     &        (dimnam(1:4).eq.dimchk(1:4))) then 
              did(k)=i
              goto 100
            endif
          enddo
 100      continue
        endif
        if (did(k).lt.0) then
c         define the dimension
          did(k)=ncddef(cdfid,dimnam,vardim(k),error)
          if (error.ne.0) goto 920
        endif
      enddo

c     define the times-array
      if (ndim.eq.4) then
c       define dimension and variable 'time'
        if (numdims.ge.4) then
          did(4)=ncdid(cdfid,'time',ierr)
          idtime=ncvid(cdfid,'time',ierr)
        else
c         this dimension must first be defined
          did(4) = ncddef (cdfid,'time',NCUNLIM,ierr)
          idtime = ncvdef (cdfid,'time',NCFLOAT,1,did(4),ierr)
        endif
      endif

c     define variable
      id=ncvdef(cdfid,varnam,NCFLOAT,ndim,did,error)
      if (error.ne.0) goto 920

c     define attributes
      do k=1,min0(ndim,3)
c       min postion
        attnam(1:1)=chrid(k)
        attnam(2:4)='min'
        attnam=attnam(1:4)
        call ncapt(cdfid,id,attnam,NCFLOAT,1,varmin(k),error)
        if (error.gt.0) goto 920
c       max position     
        attnam(1:1)=chrid(k)
        attnam(2:4)='max'
        attnam=attnam(1:4)
        call ncapt(cdfid,id,attnam,NCFLOAT,1,varmax(k),error)
        if (error.gt.0) goto 920     
c       staggering
        attnam(1:1)=chrid(k)
        attnam(2:5)='stag'
        attnam=attnam(1:5)
        call ncapt(cdfid,id,attnam,NCFLOAT,1,stag(k),error)
        if (error.gt.0) goto 920
      enddo

c     define missing data value
      call ncapt(cdfid,id,'missing_data',NCFLOAT,1,misdat,error)
      if (error.gt.0) goto 920     

c     leave define mode
      call ncendf(cdfid,error)
      if (error.gt.0) goto 920     

c     synchronyse output to disk and exit
      call ncsnc (cdfid,error)
      call ncpopt (ncopts)
      return

c     Error exits.
 900  write (6, *) '*ERROR*: When calling putcdf, the number of ',
     &             'variable dimensions must be less or equal 4.'
      call ncpopt (ncopts)
      call ncclos (cdfid, error)
      return

 920  write (6, *) '*ERROR*: An error occurred while attempting to ',
     &             'write the data file in subroutine putcdf.'
      call ncpopt (ncopts)
      call ncclos (cdfid, error)
      return
      end


      subroutine puttimes_ive(cdfid,times,ntimes,ierr)
C------------------------------------------------------------------------
C     Purpose:
C        Redefine all times on the specified NetCDF file
C     Arguments: 
C        cdfid  int  input   identifier for NetCDF file
C        times	real input   array contains all time values on the file,
C                            dimensioned at least times(ntimes)
C        ntimes int  input   number of times on the file
C        error  int  output  errorflag 
C     History:
C        Heini Wernli, ETHZ  
C        Christoph Schaer, ETHZ
C        Johannes Jenkner, ETHZ (adjustment for integer and double times)
C     Note:
C        This preliminary version does not define the times-array, but only
C        overwrites or extends an existing times-array.
C------------------------------------------------------------------------

      integer	ierr,i
      real times(*)
      integer didtim,ntimes

      integer	cdfid,idtime,nfiltim
      integer	ncdid,ncvid

      integer   vtyp,dn,nat
      integer dims(4)

      idtime=ncvid(cdfid,'time',ierr)   ! inquire id for time array
      if (ierr.ne.0) return
      didtim=ncdid(cdfid,'time',ierr)	! inquire id for time dimension
      if (ierr.ne.0) return

      call ncdinq(cdfid,didtim,'time',nfiltim,ierr)   ! inquire # of times
      if (ierr.ne.0) return

      call ncvinq(cdfid,idtime,dimnam,vtyp,dn,dims,natt,ierr)

      if (nfiltim.lt.ntimes) then
        print *,'Warning: puttimes is extending times-array'
      else if (nfiltim.gt.ntimes) then
        print *,'Warning: puttimes does not cover range of times-array'
      endif

      if (vtyp.eq.5) then
        do i=1,ntimes
          call ncvpt1(cdfid,idtime,i,times(i),ierr)
          if (ierr.ne.0) return
        enddo
      elseif (vtyp.eq.4) then ! integer version  
        do i=1,ntimes
          call ncvpt1(cdfid,idtime,i,int(times(i)),ierr) 
          if (ierr.ne.0) return
        enddo
      elseif (vtyp.eq.6) then ! double precision version
        do i=1,ntimes
          call ncvgt1(cdfid,idtime,i,dble(times(i)),ierr) 
          if (ierr.ne.0) return
        enddo        
      else
        return
      endif

      end



      subroutine gettimes_ive(cdfid,times,ntimes,ierr)
C------------------------------------------------------------------------
C     Purpose:
C        Get all times on the specified NetCDF file
C     Arguments: 
C        cdfid  int  input   identifier for NetCDF file
C        times	real output  array contains all time values on the file,
C                            dimensioned at least times(ntimes)
C        ntimes int  output  number of times on the file
C        error  int  output  errorflag 
C     History:
C        Heini Wernli, ETHZ  
C        Johannes Jenkner, ETHZ (adjustment for integer and double times)
C------------------------------------------------------------------------

      include "netcdf.inc"

      integer	ierr,i
      real times(*)
      integer didtim,ntimes

      integer	cdfid,idtime
      integer	ncopts
      character*(20) dimnam

      integer   vtyp,dn,nat
      integer dims(4)

      integer,dimension(:),allocatable :: inttimes
      double precision,dimension(:),allocatable :: doubletimes

c     Get current value of error options, and make sure netCDF-errors do 
c     not abort execution
      call ncgopt (ncopts)
      call ncpopt(NCVERBOS)

      didtim=ncdid(cdfid,'time',ierr)	! inquire id for time dimension
      if (ierr.ne.0) goto 900
      idtime=ncvid(cdfid,'time',ierr)   ! inquire id for time array
      if (ierr.ne.0) goto 900
      call ncdinq(cdfid,didtim,dimnam,ntimes,ierr)      ! inquire # of times
      if (ierr.ne.0) goto 900
  
      call ncvinq(cdfid,idtime,dimnam,vtyp,dn,dims,natt,ierr)

      if (vtyp.eq.5) then
        do i=1,ntimes
          call ncvgt1(cdfid,idtime,i,times(i),ierr) ! get times
          if (ierr.ne.0) goto 900
        enddo   
      elseif (vtyp.eq.4) then ! integer version  
        allocate(inttimes(ntimes))
        do i=1,ntimes
          call ncvgt1(cdfid,idtime,i,inttimes(i),ierr) ! get times
          if (ierr.ne.0) goto 900
        enddo
        times(1:ntimes)=real(inttimes(1:ntimes))
      elseif (vtyp.eq.6) then ! double precision version
        allocate(doubletimes(ntimes))
        do i=1,ntimes
          call ncvgt1(cdfid,idtime,i,doubletimes(i),ierr) ! get times
          if (ierr.ne.0) goto 900
        enddo
        times(1:ntimes)=real(doubletimes(1:ntimes))        
      else
        goto 900
      endif

c     normal exit
      call ncpopt (ncopts)
      return

c     error exit
 900  ntimes=1
      times(1)=0.
      call ncpopt (ncopts)
      end


      subroutine getstart_ive(cdfid,idate,ierr)
C------------------------------------------------------------------------
C     Purpose:
C	Get start date for fields on specified NetCDF file
C     Arguments:
C	cdfid	int	input	identifier for NetCDF file
C	idate	int	output	array contains date (year,month,day,time,step)
C				dimensioned as idate(5)
C	ierr	int	output	error flag
C------------------------------------------------------------------------

      include "netcdf.inc"

c     variable declarations
      integer   ierr
      integer   idate(5)
      integer   cdfid,ncopts,idvar,nvars
      integer   ndims,ngatts,recdim,i,vartyp,nvatts,vardim(4)
      character*20 vnam(100)

c     Get current value of error options, and make sure NetCDF-errors do
c     not abort execution
      call ncgopt (ncopts)
      call ncpopt (NCVERBOS)

      idvar=ncvid(cdfid,'starty',ierr)
      if (ierr.ne.0) goto 930
      call ncvgt1(cdfid,idvar,1,idate(1),ierr)
      if (ierr.ne.0) goto 920

      idvar=ncvid(cdfid,'startm',ierr)
      if (ierr.ne.0) goto 920
      call ncvgt1(cdfid,idvar,1,idate(2),ierr)
      if (ierr.ne.0) goto 920

      idvar=ncvid(cdfid,'startd',ierr)
      if (ierr.ne.0) goto920
      call ncvgt1(cdfid,idvar,1,idate(3),ierr)
      if (ierr.ne.0) goto 920

      idvar=ncvid(cdfid,'starth',ierr)
      if (ierr.ne.0) goto 920
      call ncvgt1(cdfid,idvar,1,idate(4),ierr)
      if (ierr.ne.0) goto 920

C     Starts is not defined on all files
C     Only ask for it if it exists
C     Inquire number of dimensions, variables and attributes
 
      idate(5)=0
      call ncinq(cdfid,ndims,nvars,ngatts,recdim,ierr)
      do i=1,nvars
        call ncvinq(cdfid,i,vnam(i),vartyp,ndims,vardim,nvatts,ierr)
        if (vnam(i).eq.'starts') then
          idvar=ncvid(cdfid,'starts',ierr)
          call ncvgt1(cdfid,idvar,1,idate(5),ierr)
          if (ierr.ne.0) goto 920
        endif
      enddo

c     normal exit
      call ncpopt (ncopts)
      return

c     error exit
 920  continue
      write (6, *) 'ERROR: An error occurred while attempting to ',
     &             'read the starting-time in subroutine putstart.'
 930  continue
      call ncpopt (ncopts)

      end


      subroutine putstart_ive(cdfid,idate,ierr)
C----------------------------------------------------------------------
C     Purpose:
C        Puts the 'starting-time' on the specified NetCDF file.
C     Arguments:
C        cdfid   int     input   identifier for NetCDF file
C        idate   int     input   array contains date (year,month,day,time,step)
C                                dimensioned as idate(5)
C        ierr    int     output  error flag
C------------------------------------------------------------------------

      include "netcdf.inc"

c     variable declarations
      integer   ierr,idate(5),startid(5),cdfid,ncopts,i

c     Get current value of error options, and make sure NetCDF-errors do
c     not abort execution
      call ncgopt (ncopts)
      call ncpopt (NCVERBOS)

c     define variables
      call ncredf(cdfid,ierr)
      if (ierr.ne.0) goto 920
      startid(1) = ncvdef (cdfid, 'starty', NCLONG, 0, 0, ierr)
      if (ierr.ne.0) goto 920
      startid(2) = ncvdef (cdfid, 'startm', NCLONG, 0, 0, ierr)
      if (ierr.ne.0) goto 920
      startid(3) = ncvdef (cdfid, 'startd', NCLONG, 0, 0, ierr)
      if (ierr.ne.0) goto 920
      startid(4) = ncvdef (cdfid, 'starth', NCLONG, 0, 0, ierr)
      if (ierr.ne.0) goto 920
      startid(5) = ncvdef (cdfid, 'starts', NCLONG, 0, 0, ierr)
      if (ierr.ne.0) goto 920
      call ncendf(cdfid, ierr)
      if (ierr.ne.0) goto 920

c     store variables
      do i=1,5
        call ncvpt1(cdfid,startid(i),1,idate(i),ierr)
        if (ierr.ne.0) goto 920
      enddo

c     synchronyse output to disk, revert to previous error-mode, and exit
      call ncsnc (cdfid,ierr)
      call ncpopt (ncopts)
      return

c     error exit
 920  write (6, *) 'ERROR: An error occurred while attempting to ',
     &             'write the starting-time in subroutine putstart.'
      call ncpopt (ncopts)
      call ncclos (cdfid, ierr)

      end


      subroutine getgrid_ive(cdfid,dx,dy,ierr)
C------------------------------------------------------------------------
C     Purpose:
C       Get grid increments for fields on specified NetCDF file
C     Arguments:
C       cdfid   int     input   identifier for NetCDF file
C	dx	real	output	grid increment along latitude
C	dy	real	output	grid increment along longitude
C       ierr    int     output  error flag
C------------------------------------------------------------------------

      use netcdf

      integer   cdfid
      real      dx
      real      dy
      integer   ierror

      ierror  = nf90_get_att(cdfid, nf90_global, "dellon", dx)
      IF (ierror /= nf90_NoErr) RETURN
      ierror  = nf90_get_att(cdfid, nf90_global, "dellat", dy)
      IF (ierror /= nf90_NoErr) RETURN

      end



      subroutine getpole_ive(cdfid,pollon,pollat,polgam,ierr)
C------------------------------------------------------------------------
C     Purpose:
C       Get physical coordinates of pole of coordinate system
C     Arguments:
C       cdfid   int     input   identifier for NetCDF file
C	pollon	real	output	longitude of pole
C	pollat	real	output	latitude of pole
C       ierr    int     output  error flag
C------------------------------------------------------------------------

      use netcdf

      integer   cdfid
      real      pollon
      real      pollat
      real      polgam
      integer   ierror

      ierror  = nf90_get_att(cdfid,nf90_global, "pollon", pollon)
      IF(ierror /= nf90_NoErr) pollon = 0.
      ierror  = nf90_get_att(cdfid,nf90_global, "pollat", pollat)
      IF(ierror /= nf90_NoErr) pollat = 90.

       polgam = 0.

c      print*,' WARNING: polgam not supported in IVE mode -> set to 0 '
c      print*

      end


      subroutine getcfn_ive(cdfid,cfn,ierr)
C------------------------------------------------------------------------
C     Purpose:
C       Get name of constants file
C     Arguments:
C       cdfid   int     input   identifier for NetCDF file
C       cfn     char    output  name of constants file
C       ierr    int     output  error flag
C------------------------------------------------------------------------

      include "netcdf.inc"

      integer   ierr
      integer   cdfid,lenstr
      character*80 cfn

      lenstr=80
      call ncagtc(cdfid,NCGLOBAL,"constants_file_name",cfn,lenstr,ierr)
      if (ierr.ne.0) write(*,*)'error in SR getcfn'

      end


      subroutine getvars_ive(cdfid,nvars,vnam,ierr)
C------------------------------------------------------------------------
 
C     Opens the NetCDF file 'filnam' and returns its identifier cdfid.
 
C     filnam    char    input   name of NetCDF file to open
C     nvars     int     output  number of variables on file
C     vnam	char	output  array with variable names
C     ierr      int     output  error flag
C------------------------------------------------------------------------

      include "netcdf.inc"
 
      integer   cdfid,ierr,nvars
      character*(*) vnam(*)

      integer	ndims,ngatts,recdim,i,vartyp,nvatts,vardim(4)
 
      call ncpopt(NCVERBOS)

C     Inquire number of dimensions, variables and attributes
 
      call ncinq(cdfid,ndims,nvars,ngatts,recdim,ierr)
 
C     Inquire variable names from NetCDF file
 
      do i=1,nvars
        call ncvinq(cdfid,i,vnam(i),vartyp,ndims,vardim,nvatts,ierr)
      enddo
 
      return
      end

      subroutine cdfopn_ive(filnam,cdfid,ierr)
C------------------------------------------------------------------------

C     Opens the NetCDF file 'filnam' and returns its identifier cdfid.

C     filnam    char    input   name of NetCDF file to open
C     cdfid     int     output  identifier of NetCDF file
C     ierr	int	output  error flag
C------------------------------------------------------------------------

      include "netcdf.inc"

      integer 	cdfid,ierr
      character*(*) filnam

      call ncpopt(NCVERBOS)
      cdfid=ncopn(trim(filnam),NCNOWRIT,ierr)

      return
      end


      subroutine cdfwopn_ive(filnam,cdfid,ierr)
C------------------------------------------------------------------------

C     Opens the NetCDF file 'filnam' and returns its identifier cdfid.

C     filnam    char    input   name of NetCDF file to open
C     cdfid     int     output  identifier of NetCDF file
C     ierr      int     output  error flag
C------------------------------------------------------------------------

      include "netcdf.inc"

      integer   cdfid,ierr
      character*(*) filnam

      call ncpopt(NCVERBOS)
      cdfid=ncopn(trim(filnam),NCWRITE,ierr)

      return
      end
c***********************************************************************
c MASTER-netCDF
c***********************************************************************

c     ------- Format specification -------------------------------------

      subroutine getformat (mode,cdfid)

      character*80 mode
      integer      cdfid
      
c     Reset the mode
      mode = ''
      call getenv('NETCDF_FORMAT',mode)
      if ( mode.eq.'' ) then
        mode = 'IVE'
      endif

      end


c     ------- Interface routines ---------------------------------------


      subroutine clscdf (cdfid, error)

      integer      cdfid
      integer      error
      character*80 mode

      call getformat(mode,cdfid)

      if ( mode.eq.'IVE' ) then 
         call clscdf_ive (cdfid, error)
         
      elseif ( mode.eq.'CF' ) then
         call clscdf_cf (cdfid, error)
         
      elseif ( mode.eq.'MCH' ) then
         call clscdf_mch (cdfid, error)

      endif

      end


      subroutine crecdf (filnam, cdfid, phymin, phymax, ndim,
     &                   nlon, nlat, nlev, pollon, pollat, error) 

      character *(*) filnam
      integer        cdfid
      real           phymin(*)
      real           phymax(*)
      integer        ndim
      integer        nlon
      integer        nlat
      integer        nlev
      real           pollon
      real           pollat
      integer        error
      character*80 mode

      call getformat(mode,cdfid)
     
      if ( mode.eq.'IVE' ) then
           call crecdf_ive (filnam, cdfid, phymin, phymax, ndim,
     &                      nlon, nlat, nlev, pollon, pollat, error)
         
      elseif ( mode.eq.'CF' ) then
           call crecdf_cf  (filnam, cdfid, phymin, phymax, ndim,
     &                      nlon, nlat, nlev, pollon, pollat, error) 
         
      elseif ( mode.eq.'MCH' ) then
           call crecdf_mch  (filnam, cdfid, phymin, phymax, ndim,
     &                      nlon, nlat, nlev, pollon, pollat, error) 
      
      endif

      end


      subroutine getdef (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)

      integer        cdfid
      character *(*) varnam
      integer        ndim
      real           misdat
      integer        vardim(*)
      real           varmin(*)
      real           varmax(*)
      real           stag(*)
      integer        error
      character*80   mode
 
      call getformat(mode,cdfid)
 
      if ( mode.eq.'IVE' ) then
         call getdef_ive (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)
         
      elseif ( mode.eq.'CF' ) then
         call getdef_cf  (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)
     
      elseif ( mode.eq.'MCH' ) then
         call getdef_mch  (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)
     
      endif

      end


      subroutine getdat (cdfid, varnam, time, level, dat, error)

      integer        cdfid
      character*(*)  varnam
      real           time
      integer        level
      real           dat(*)
      integer        error
      character*80   mode

      call getformat(mode,cdfid)
     
      if ( mode.eq.'IVE' ) then
         call getdat_ive (cdfid, varnam, time, level, dat, error)
         
      elseif ( mode.eq.'CF' ) then
         call getdat_cf  (cdfid, varnam, time, level, dat, error)
         
      elseif ( mode.eq.'MCH' ) then
         call getdat_mch  (cdfid, varnam, time, level, dat, error)
      
      endif

      end


      subroutine putdat (cdfid, varnam, time, level, dat, error)

      integer        cdfid
      character*(*)  varnam
      real           time
      integer        level
      real 	     dat(*)
      integer        error
      character*80   mode

      call getformat(mode,cdfid)

      if ( mode.eq.'IVE' ) then         
         call putdat_ive (cdfid, varnam, time, level, dat, error)
         
      elseif ( mode.eq.'CF' ) then
         call putdat_cf  (cdfid, varnam, time, level, dat, error)
         
      elseif ( mode.eq.'MCH' ) then
         call putdat_mch  (cdfid, varnam, time, level, dat, error)

      endif

      end


      subroutine putdef (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)

      integer        cdfid
      character*(*)  varnam
      integer        ndim
      real           misdat
      integer        vardim(*)
      real           varmin(*)
      real           varmax(*)
      real           stag(*)
      integer        error
      character*80   mode

      call getformat(mode,cdfid)

      if ( mode.eq.'IVE' ) then
         call putdef_ive (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)      
         
      elseif ( mode.eq.'CF' ) then
         call putdef_cf  (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)
     
      elseif ( mode.eq.'MCH' ) then
         call putdef_mch  (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)
     
      endif
    
      end


      subroutine puttimes (cdfid,times,ntimes,ierr)

      integer        cdfid
      real           times(*)
      integer        ntimes
      integer	     ierr
      character*80   mode

      call getformat(mode,cdfid)

      if ( mode.eq.'IVE' ) then
         call puttimes_ive (cdfid,times,ntimes,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call puttimes_cf  (cdfid,times,ntimes,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call puttimes_mch  (cdfid,times,ntimes,ierr)   
      
      endif

      end



      subroutine gettimes (cdfid,times,ntimes,ierr)

      integer        cdfid
      real           times(*)
      integer        ntimes
      integer	     ierr
      character*80   mode

      call getformat(mode,cdfid)

      if ( mode.eq.'IVE' ) then
         call gettimes_ive (cdfid,times,ntimes,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call gettimes_cf  (cdfid,times,ntimes,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call gettimes_mch  (cdfid,times,ntimes,ierr)
         
      endif

      end


      subroutine wricst (cstnam,datar,aklev,bklev,aklay,bklay,stdate)

      character*80   cstnam
      integer        datar(14)
      integer        nchar,maxlev
      parameter      (nchar=20,maxlev=32)
      real	         aklev(maxlev)
      real           bklev(maxlev)
      real           aklay(maxlev)
      real           bklay(maxlev)
      integer	     stdate(5)
      character*80   mode

      print*,'<wricst> not available for COSMO files'
      stop

      end

 
      subroutine getlevs (cstid,nlev,aklev,bklev,aklay,bklay,error)
      
      integer        cstid
      integer        nlev
      real           aklev(nlev)
      real           bklev(nlev)
      real           aklay(nlev)
      real           bklay(nlev)      
      integer        error
      character*80   mode

      print*,'<getlevs> not available for COSMO files'
      stop

      end


      subroutine getstart (cdfid,idate,ierr)
      
      integer        cdfid
      integer        idate(5)
      integer        ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call getstart_ive (cdfid,idate,ierr)
         
      elseif ( mode.eq.'CF' ) then
         print*,'<getstart> doesnt support CF format -> STOP'
      	 stop
         
      elseif ( mode.eq.'MCH' ) then
         print*,'<getstart> doesnt support MCH format -> STOP'
      	 stop
       
         
      endif

      end


      subroutine putstart (cdfid,idate,ierr)

      integer        cdfid
      integer        idate(5)
      integer        ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call putstart_ive (cdfid,idate,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call putstart_cf  (cdfid,idate,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call putstart_mch  (cdfid,idate,ierr)
         
      endif

      end


      subroutine getgrid (cdfid,dx,dy,ierr)

      integer        cdfid
      real           dx
      real           dy
      integer        ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call getgrid_ive (cdfid,dx,dy,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call getgrid_cf (cdfid,dx,dy,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call getgrid_mch (cdfid,dx,dy,ierr)
         
      endif

      end


      subroutine getpole (cdfid,pollon,pollat,polgam,ierr)

      integer        cdfid
      real           pollon
      real           pollat
      real           polgam
      integer        ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call getpole_ive (cdfid,pollon,pollat,polgam,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call getpole_cf  (cdfid,pollon,pollat,polgam,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call getpole_mch  (cdfid,pollon,pollat,polgam,ierr)
         
      endif

      end


      subroutine getcfn (cdfid,cfn,ierr)

      integer        ierr
      integer        cdfid
      character*80   cfn
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call getcfn_ive (cdfid,cfn,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call getcfn_cf  (cdfid,cfn,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call getcfn_mch  (cdfid,cfn,ierr)        
      
      endif

      end


      subroutine getvars (cdfid,nvars,vnam,ierr)

      integer        cdfid
      integer        nvars
      character*(*)  vnam(*)
      integer        ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call getvars_ive (cdfid,nvars,vnam,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call getvars_cf  (cdfid,nvars,vnam,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call getvars_mch  (cdfid,nvars,vnam,ierr)
         
      endif

      end


      subroutine cdfopn (filnam,cdfid,ierr)

      integer 	     cdfid
      character*(*)  filnam
      integer 	     ierr
      character*80   mode

      call getformat(mode,cdfid)   

      if ( mode.eq.'IVE' ) then
         call cdfopn_ive (filnam,cdfid,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call cdfopn_cf (filnam,cdfid,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call cdfopn_mch (filnam,cdfid,ierr)
         
      endif

      end


      subroutine cdfwopn (filnam,cdfid,ierr)

      integer 	     cdfid
      character*(*)  filnam
      integer 	     ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call cdfwopn_ive (filnam,cdfid,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call cdfwopn_cf (filnam,cdfid,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call cdfwopn_mch (filnam,cdfid,ierr)
      
      endif

      end


  
