c***********************************************************************
c MASTER-netCDF
c***********************************************************************

c     ------- Format specification -------------------------------------

      subroutine getformat (mode,cdfid)

      character*80 mode
      integer      cdfid
      
c     Reset the mode
      mode = ''
      call getenv('NETCDF_FORMAT',mode)
      if ( mode.eq.'' ) then
        mode = 'IVE'
      endif

      end


c     ------- Interface routines ---------------------------------------


      subroutine clscdf (cdfid, error)

      integer      cdfid
      integer      error
      character*80 mode

      call getformat(mode,cdfid)

      if ( mode.eq.'IVE' ) then 
         call clscdf_ive (cdfid, error)
         
      elseif ( mode.eq.'CF' ) then
         call clscdf_cf (cdfid, error)
         
      elseif ( mode.eq.'MCH' ) then
         call clscdf_mch (cdfid, error)

      endif

      end


      subroutine crecdf (filnam, cdfid, phymin, phymax, ndim,
     &                   nlon, nlat, nlev, pollon, pollat, error) 

      character *(*) filnam
      integer        cdfid
      real           phymin(*)
      real           phymax(*)
      integer        ndim
      integer        nlon
      integer        nlat
      integer        nlev
      real           pollon
      real           pollat
      integer        error
      character*80 mode

      call getformat(mode,cdfid)
     
      if ( mode.eq.'IVE' ) then
           call crecdf_ive (filnam, cdfid, phymin, phymax, ndim,
     &                      nlon, nlat, nlev, pollon, pollat, error)
         
      elseif ( mode.eq.'CF' ) then
           call crecdf_cf  (filnam, cdfid, phymin, phymax, ndim,
     &                      nlon, nlat, nlev, pollon, pollat, error) 
         
      elseif ( mode.eq.'MCH' ) then
           call crecdf_mch  (filnam, cdfid, phymin, phymax, ndim,
     &                      nlon, nlat, nlev, pollon, pollat, error) 
      
      endif

      end


      subroutine getdef (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)

      integer        cdfid
      character *(*) varnam
      integer        ndim
      real           misdat
      integer        vardim(*)
      real           varmin(*)
      real           varmax(*)
      real           stag(*)
      integer        error
      character*80   mode
 
      call getformat(mode,cdfid)
 
      if ( mode.eq.'IVE' ) then
         call getdef_ive (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)
         
      elseif ( mode.eq.'CF' ) then
         call getdef_cf  (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)
     
      elseif ( mode.eq.'MCH' ) then
         call getdef_mch  (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)
     
      endif

      end


      subroutine getdat (cdfid, varnam, time, level, dat, error)

      integer        cdfid
      character*(*)  varnam
      real           time
      integer        level
      real           dat(*)
      integer        error
      character*80   mode

      call getformat(mode,cdfid)
     
      if ( mode.eq.'IVE' ) then
         call getdat_ive (cdfid, varnam, time, level, dat, error)
         
      elseif ( mode.eq.'CF' ) then
         call getdat_cf  (cdfid, varnam, time, level, dat, error)
         
      elseif ( mode.eq.'MCH' ) then
         call getdat_mch  (cdfid, varnam, time, level, dat, error)
      
      endif

      end


      subroutine putdat (cdfid, varnam, time, level, dat, error)

      integer        cdfid
      character*(*)  varnam
      real           time
      integer        level
      real 	     dat(*)
      integer        error
      character*80   mode

      call getformat(mode,cdfid)

      if ( mode.eq.'IVE' ) then         
         call putdat_ive (cdfid, varnam, time, level, dat, error)
         
      elseif ( mode.eq.'CF' ) then
         call putdat_cf  (cdfid, varnam, time, level, dat, error)
         
      elseif ( mode.eq.'MCH' ) then
         call putdat_mch  (cdfid, varnam, time, level, dat, error)

      endif

      end


      subroutine putdef (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)

      integer        cdfid
      character*(*)  varnam
      integer        ndim
      real           misdat
      integer        vardim(*)
      real           varmin(*)
      real           varmax(*)
      real           stag(*)
      integer        error
      character*80   mode

      call getformat(mode,cdfid)

      if ( mode.eq.'IVE' ) then
         call putdef_ive (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)      
         
      elseif ( mode.eq.'CF' ) then
         call putdef_cf  (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)
     
      elseif ( mode.eq.'MCH' ) then
         call putdef_mch  (cdfid, varnam, ndim, misdat, 
     &                   vardim, varmin, varmax, stag, error)
     
      endif
    
      end


      subroutine puttimes (cdfid,times,ntimes,ierr)

      integer        cdfid
      real           times(*)
      integer        ntimes
      integer	     ierr
      character*80   mode

      call getformat(mode,cdfid)

      if ( mode.eq.'IVE' ) then
         call puttimes_ive (cdfid,times,ntimes,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call puttimes_cf  (cdfid,times,ntimes,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call puttimes_mch  (cdfid,times,ntimes,ierr)   
      
      endif

      end



      subroutine gettimes (cdfid,times,ntimes,ierr)

      integer        cdfid
      real           times(*)
      integer        ntimes
      integer	     ierr
      character*80   mode

      call getformat(mode,cdfid)

      if ( mode.eq.'IVE' ) then
         call gettimes_ive (cdfid,times,ntimes,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call gettimes_cf  (cdfid,times,ntimes,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call gettimes_mch  (cdfid,times,ntimes,ierr)
         
      endif

      end


      subroutine wricst (cstnam,datar,aklev,bklev,aklay,bklay,stdate)

      character*80   cstnam
      integer        datar(14)
      integer        nchar,maxlev
      parameter      (nchar=20,maxlev=32)
      real	         aklev(maxlev)
      real           bklev(maxlev)
      real           aklay(maxlev)
      real           bklay(maxlev)
      integer	     stdate(5)
      character*80   mode

      print*,'<wricst> not available for COSMO files'
      stop

      end

 
      subroutine getlevs (cstid,nlev,aklev,bklev,aklay,bklay,error)
      
      integer        cstid
      integer        nlev
      real           aklev(nlev)
      real           bklev(nlev)
      real           aklay(nlev)
      real           bklay(nlev)      
      integer        error
      character*80   mode

      print*,'<getlevs> not available for COSMO files'
      stop

      end


      subroutine getstart (cdfid,idate,ierr)
      
      integer        cdfid
      integer        idate(5)
      integer        ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call getstart_ive (cdfid,idate,ierr)
         
      elseif ( mode.eq.'CF' ) then
         print*,'<getstart> doesnt support CF format -> STOP'
      	 stop
         
      elseif ( mode.eq.'MCH' ) then
         print*,'<getstart> doesnt support MCH format -> STOP'
      	 stop
       
         
      endif

      end


      subroutine putstart (cdfid,idate,ierr)

      integer        cdfid
      integer        idate(5)
      integer        ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call putstart_ive (cdfid,idate,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call putstart_cf  (cdfid,idate,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call putstart_mch  (cdfid,idate,ierr)
         
      endif

      end


      subroutine getgrid (cdfid,dx,dy,ierr)

      integer        cdfid
      real           dx
      real           dy
      integer        ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call getgrid_ive (cdfid,dx,dy,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call getgrid_cf (cdfid,dx,dy,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call getgrid_mch (cdfid,dx,dy,ierr)
         
      endif

      end


      subroutine getpole (cdfid,pollon,pollat,polgam,ierr)

      integer        cdfid
      real           pollon
      real           pollat
      real           polgam
      integer        ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call getpole_ive (cdfid,pollon,pollat,polgam,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call getpole_cf  (cdfid,pollon,pollat,polgam,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call getpole_mch  (cdfid,pollon,pollat,polgam,ierr)
         
      endif

      end


      subroutine getcfn (cdfid,cfn,ierr)

      integer        ierr
      integer        cdfid
      character*80   cfn
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call getcfn_ive (cdfid,cfn,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call getcfn_cf  (cdfid,cfn,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call getcfn_mch  (cdfid,cfn,ierr)        
      
      endif

      end


      subroutine getvars (cdfid,nvars,vnam,ierr)

      integer        cdfid
      integer        nvars
      character*(*)  vnam(*)
      integer        ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call getvars_ive (cdfid,nvars,vnam,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call getvars_cf  (cdfid,nvars,vnam,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call getvars_mch  (cdfid,nvars,vnam,ierr)
         
      endif

      end


      subroutine cdfopn (filnam,cdfid,ierr)

      integer 	     cdfid
      character*(*)  filnam
      integer 	     ierr
      character*80   mode

      call getformat(mode,cdfid)   

      if ( mode.eq.'IVE' ) then
         call cdfopn_ive (filnam,cdfid,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call cdfopn_cf (filnam,cdfid,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call cdfopn_mch (filnam,cdfid,ierr)
         
      endif

      end


      subroutine cdfwopn (filnam,cdfid,ierr)

      integer 	     cdfid
      character*(*)  filnam
      integer 	     ierr
      character*80   mode

      call getformat(mode,cdfid)
  
      if ( mode.eq.'IVE' ) then
         call cdfwopn_ive (filnam,cdfid,ierr)
         
      elseif ( mode.eq.'CF' ) then
         call cdfwopn_cf (filnam,cdfid,ierr)
         
      elseif ( mode.eq.'MCH' ) then
         call cdfwopn_mch (filnam,cdfid,ierr)
      
      endif

      end


  
