c***********************************************************************
c CF-netCDF / Reading
c***********************************************************************

c     -------------------------------------------------------------------
c     cdfopn
c     -------------------------------------------------------------------

      subroutine cdfopn_mch (filnam,cdfid,ierror)

      use netcdf
      
      character*(*) filnam
      integer 	    cdfid
      integer       ierror

      ierror = NF90_OPEN(TRIM(filnam),nf90_nowrite, cdfid)
      IF ( ierror /= nf90_NoErr ) PRINT *,NF90_STRERROR(ierror)

      return
      end


c     -------------------------------------------------------------------
c     clscdf
c     -------------------------------------------------------------------

      subroutine clscdf_mch (cdfid, ierror)

      use netcdf
      
      integer      cdfid
      integer      ierror

      ierror = NF90_CLOSE(cdfid)
      IF( ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      end


c     -------------------------------------------------------------------
c     getdef
c     -------------------------------------------------------------------

      subroutine getdef_mch (cdfid, varnam, ndim, misdat, 
     &                      vardim, varmin, varmax, stag, ierror)

      use netcdf

      integer        cdfid
      character *(*) varnam
      integer        ndim
      real           misdat
      integer        vardim(4)
      real           varmin(4)
      real           varmax(4)
      real           stag(4)
      integer        ierror

      real           lat(5000)
      real           lon(5000)
      real           lev(5000)
      real           tim(5000)
      real           dlon,dlat
      integer        varid
      integer        lonid
      integer        latid
      integer        levid
      integer        timid
      integer        dimids (nf90_max_var_dims)
      character*80   dimname(nf90_max_var_dims)
      integer        nlon
      integer        nlat
      integer        nlev
      integer        ntim
      character*(80) attname
      integer        i

c     Get <ndim>, <dimids> and <dimname>
      ierror = NF90_INQ_VARID(cdfid,varnam,varid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_variable(cdfid, varid, ndims  = ndim)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_variable(cdfid, varid, 
     >                                   dimids = dimids(1:ndim))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      do i=1,ndim
         ierror = nf90_inquire_dimension(cdfid, dimids(i), 
     >                               name = dimname(i) )
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      enddo

c     Check which dimensions are available
      lonid = -999
      latid = -999
      levid = -999
      timid = -999
      do i=1,ndim
        if ( dimname(i).eq.'x_1' ) then
           ierror = nf90_inq_dimid(cdfid,dimname(i), lonid)
           IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        endif
        if ( dimname(i).eq.'y_1' ) then
           ierror = nf90_inq_dimid(cdfid,dimname(i), latid)
           IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        endif
        if ( ( dimname(i).eq.'z_1' ).or.
     >       ( dimname(i).eq.'z_2' ).or.
     >       ( dimname(i).eq.'z_3' ).or.
     >       ( dimname(i).eq.'z_4' ).or.
     >       ( dimname(i).eq.'z_5' ).or.
     >       ( dimname(i).eq.'z_6' ) ) 
     >  then
           ierror = nf90_inq_dimid(cdfid,dimname(i), levid)
           IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        endif
        if ( dimname(i).eq.'time' ) then
           ierror = nf90_inq_dimid(cdfid,'time', timid)
           IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        endif
      enddo  
 
c     Read lon and set <vardim(1)>, <varmin(1)>, <varmax(1)>, <stag(1)>
      if ( lonid.ne.-999 ) then
        ierror = nf90_inquire_dimension(cdfid, lonid, len = nlon)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_INQ_VARID(cdfid,'x_1',varid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_GET_VAR(cdfid,varid,lon(1:nlon))
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
        vardim(1) = nlon
        varmin(1) = lon( 1    )
        varmax(1) = lon( nlon )
        stag  (1) = 0.
      else
        vardim(1) = -999
        varmin(1) = 0.
        varmax(1) = 0.
        stag  (1) = 0.
      endif
      
c     Read lat and set <vardim(2)>, <varmin(2)>, <varmax(2)>, <stag(2)>
      if ( latid.ne.-999 ) then
        ierror = nf90_inquire_dimension(cdfid, latid, len = nlat)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_INQ_VARID(cdfid,'y_1',varid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_GET_VAR(cdfid,varid,lat(1:nlat))
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
        vardim(2) = nlat
        varmin(2) = lat( 1    )
        varmax(2) = lat( nlat )
        stag  (2) = 0.
      else
        vardim(2) = -999
        varmin(2) = 0.
        varmax(2) = 0.
        stag  (2) = 0.
      endif
      
c     Read lev and set <vardim(3)>, <varmin(3)>, <varmax(3)>, <stag(3)>
      if ( levid.ne.-999 ) then
        ierror = nf90_inquire_dimension(cdfid, levid, len = nlev)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        vardim(3) = nlev
        varmin(3) = 1
        varmax(3) = nlev
        stag  (3) = 0.
        if ( (trim(varnam) == 'W'  ) .OR. 
     >       (trim(varnam) == 'HHL') .OR.
     >       (trim(varnam) == 'TKE' ))
     >  then
             stag(3) = -0.5
        endif
      else
        vardim(3) = -999
        varmin(3) = 0.
        varmax(3) = 0.
        stag  (3) = 0.
      endif

c     Read time and set <vardim(4)>, <varmin(4)>, <varmax(4)>, <stag(4)>
      if ( timid.ne.-999 ) then
        ierror = nf90_inquire_dimension(cdfid, timid, len = ntim)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_INQ_VARID(cdfid,'time',varid)
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
        ierror = NF90_GET_VAR(cdfid,varid,tim(1:ntim))
        IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
        vardim(4) = ntim
        varmin(4) = tim( 1    )
        varmax(4) = tim( ntim )
        stag  (4) = 0.0
      else
        vardim(4) = -999
        varmin(4) = 0.
        varmax(4) = 0.
        stag  (4) = 0.
      endif  

c     Read missing data value <misdat>
      ierror = NF90_INQ_VARID(cdfid,varnam,varid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_get_att(cdfid, varid, "_FillValue", misdat)
      IF(ierror /= nf90_NoErr) THEN
         misdat = -999.99
         ierror = 0
      ENDIF

c     Do some checks
      if ( varmax(1).lt.varmin(1) ) then
         print*,' Error: Wrong order of longitudes'
         ierror = 1
         return
      endif
      if ( varmax(2).lt.varmin(2) ) then
         print*,' Error: Wrong order of latitudes'
         ierror = 1
         return
      endif

      end

c     -------------------------------------------------------------------
c     getgrid
c     -------------------------------------------------------------------

      subroutine getgrid_mch (cdfid,dx,dy,ierror)

      use netcdf

      integer   cdfid
      real      dx
      real      dy
      integer   ierror

      integer   lonid
      integer   latid
      real      lat(5000)
      real      lon(5000)
      integer   nlon
      integer   nlat

c     Get the ID for <longitude>, <latitude>, <level> and <time>
      ierror = nf90_inq_dimid(cdfid, 'x_1', lonid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, lonid, len = nlon)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_INQ_VARID(cdfid, 'x_1' ,lonid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inq_dimid(cdfid, 'y_1', latid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, latid, len = nlat)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_INQ_VARID(cdfid, 'y_1' ,latid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Read lon and calculate <dx>
      ierror = NF90_GET_VAR(cdfid,lonid,lon(1:nlon))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      dx = ( lon(nlon) - lon(1) ) / ( real(nlon) - 1.)

c     Read lat and calculate <dy>
      ierror = NF90_GET_VAR(cdfid,latid,lat(1:nlat))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
      dy = ( lat(nlat) - lat(1) ) / ( real(nlat) - 1.)

      end


c     -------------------------------------------------------------------
c     getpole
c     -------------------------------------------------------------------

      subroutine getpole_mch (cdfid,pollon,pollat,polgam,ierror)

      use netcdf

      integer   cdfid, varid
      real      pollon, pollat, polgam
      integer   ierror

      ierror = nf90_inq_varid(cdfid, "grid_mapping_1", varid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      
      ierror  = nf90_get_att(cdfid, varid,
     >                       "grid_north_pole_longitude", pollon)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_get_att(cdfid, varid,
     >                       "grid_north_pole_latitude", pollat)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_get_att(cdfid, varid,
     >                       "north_pole_grid_longitude", polgam)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      end

c     -------------------------------------------------------------------
c     getcfn
c     -------------------------------------------------------------------

      subroutine getcfn_mch (cdfid,cfn,ierror)
      
      use netcdf

      integer      cdfid
      character*80 cfn
      integer      ierror

      cfn='LMCONSTANTS'

      end


c     -------------------------------------------------------------------
c     getdat
c     -------------------------------------------------------------------

      subroutine getdat_mch (cdfid, varnam, time, level, dat, ierror)

      use netcdf

      integer       cdfid
      character*(*) varnam
      real          time
      integer       level
      real          dat(*)
      integer       ierror

      integer       varid
      integer       ndim
      real          misdat
      integer       vardim(4)
      real          varmin(4)
      real          varmax(4)
      real          stag(4)
      integer       ntim
      real          tim(5000)
      integer       itime
      integer       corner(4)
      integer       count (4)
      real,allocatable, dimension (:,:,:) :: tmp3
      integer       stat
      integer       i,j,k
      character*80  dir

c     Get variable definition
      call getdef_mch (cdfid, varnam, ndim, misdat, 
     &                vardim, varmin, varmax, stag, ierror)

c     Get list of times and get appropriate time index
      if ( vardim(4).gt.0 ) then
         call gettimes_mch (cdfid,tim,ntim,ierror)
         itime = 0
         do i=1,ntim
            if ( tim(i).eq.time) itime = i
         enddo
         if ( itime.eq.0 ) then
            ierror = 1
            print*,'Error: Unknown time in getdat'
            return
         endif
      else 
         itime  = 1
         ntim   = 0
      endif

c     Select box in netCDF file
      count(1) = -999
      count(2) = -999
      count(3) = -999
      count(4) = -999
      if ( vardim(1).gt.0 ) then
         corner(1) = 1
         count (1) = vardim(1)
      else
         print*,' ERROR: lon missing for ',trim(varnam)
         stop
      endif
      if ( vardim(2).gt.0 ) then
         corner(2) = 1
         count (2) = vardim(2)
      else
         print*,' ERROR: lat missing for ',trim(varnam)
         stop
      endif
      if ( (vardim(3).gt.0).and.(level.eq.0) ) then
         corner(3) = 1
         count (3) = vardim(3)
      elseif ( (vardim(3).gt.0).and.(level.ge.1) ) then
         corner(3) = vardim(3) - level + 1
         count (3) = 1
      endif
      if ( vardim(4).gt.0 ) then
         corner(4) = itime
         count (4) = 1
      endif
          
c     Read data
      ierror = NF90_INQ_VARID(cdfid,varnam,varid)
      IF (ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      if ( (vardim(3).gt.0).and.(vardim(4).gt.0) ) then
        allocate(tmp3(count(1),count(2),count(3)),stat=stat)
        ierror = nf90_get_var(cdfid,varid,tmp3,
     >           start = (/ corner(1),corner(2),corner(3),corner(4) /),
     >           count = (/ count (1),count (2),count (3),count (4) /) ) 
     
      elseif ( (vardim(3).lt.0).and.(vardim(4).gt.0) ) then
        allocate(tmp3(count(1),count(2),1),stat=stat)
        ierror = nf90_get_var(cdfid,varid,tmp3,
     >           start = (/ corner(1),corner(2),corner(4) /),
     >           count = (/ count (1),count (2),count (4) /) ) 
     
      elseif ( (vardim(3).gt.0).and.(vardim(4).lt.0) ) then
        allocate(tmp3(count(1),count(2),count(3)),stat=stat)
        ierror = nf90_get_var(cdfid,varid,tmp3,
     >           start = (/ corner(1),corner(2),corner(3) /),
     >           count = (/ count (1),count (2),count (3) /) ) 
      
      elseif ( (vardim(3).lt.0).and.(vardim(4).lt.0) ) then
        allocate(tmp3(count(1),count(2),1),stat=stat)
        ierror = nf90_get_var(cdfid,varid,tmp3,
     >           start = (/ corner(1),corner(2) /),
     >           count = (/ count (1),count (2) /) ) 
     
      endif

c     Copy data to output array: change orientation of vertical axis
      do i=1,count(1)
         do j=1,count(2)
            if ( count(3).gt.1) then

               do k=1,count(3)
                     dat(i+count(1)*(j-1)+count(1)*count(2)*(k-1)) = 
     >                                      tmp3(i,j,count(3)-k+1)
               enddo

            else 
               dat(i+count(1)*(j-1)) = tmp3(i,j,1)
            endif

         enddo
      enddo
               
c     Deallocate memory
      deallocate(tmp3,stat=stat)

      end

c     -------------------------------------------------------------------
c     gettimes
c     -------------------------------------------------------------------

      subroutine gettimes_mch (cdfid,times,ntimes,ierror)

      use netcdf
      
      integer cdfid
      real    times(*)
      integer ntimes
      integer ierror
      
      integer timid

c     Get all times 
      ierror = nf90_inq_dimid(cdfid,'time', timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, timid, len = ntimes)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_INQ_VARID(cdfid,'time',timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_GET_VAR(cdfid,timid,times(1:ntimes))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end


c     -------------------------------------------------------------------
c     getvars
c     -------------------------------------------------------------------

      subroutine getvars_mch (cdfid,nvars,vnam,ierror)

      use netcdf
      
      integer           cdfid
      integer           nvars
      character*80      vnam(*)
      integer           ierror

      integer           i

c     Get number of variables
      ierror = nf90_inquire(cdfid, nVariables = nvars)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      
c     Get variable names
      do i=1,nvars
          ierror = nf90_inquire_variable(cdfid, i, name = vnam(i) )
          IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      enddo

      return
      end


c***********************************************************************
c CF-netCDF / Writing
c***********************************************************************

c     -------------------------------------------------------------------
c     crecdf
c     -------------------------------------------------------------------

      subroutine crecdf_mch (filnam, cdfid, phymin, phymax, ndim, 
     &                      nlon, nlat, nlev, pollon, pollat, ierror) 
      
      use netcdf
      
      character*(* ) filnam
      integer        cdfid
      real           phymin(4), phymax(4)
      real           pollon, pollat
      real           val(1000)
      integer        ndim, dimid(4), varid(4)
      integer        nlon, nlat, nlev
      integer        ierror, i

c     Create the netCDF file
      ierror = nf90_create(path  = filnam, 
     >                     cmode = nf90_clobber, 
     >                     ncid  = cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Global attributes
      ierror = nf90_put_att(cdfid, nf90_global, 
     >                            "Conventions", "CF")
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, nf90_global, 
     >                             "institution", 'IACETH')
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     longitude dimension
      ierror = nf90_def_dim(cdfid, "rlon", nlon, dimid(1))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_def_var(cdfid, "rlon",
     >                      NF90_FLOAT,dimid(1),varid(1))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(1), 
     >                      "long_name", "rotated longitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(1), 
     >                      "units", "degrees") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(1), 
     >                      "standard_name", "longitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     latitude dimension
      ierror = nf90_def_dim(cdfid, "rlat", nlat, dimid(2))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_def_var(cdfid, "rlat",
     >                      NF90_FLOAT,dimid(2),varid(2))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(2), 
     >                      "long_name", "rotated latitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(2), 
     >                      "units", "degrees") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(2), 
     >                      "standard_name", "latitude") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     level dimension
      ierror = nf90_def_dim(cdfid, "level", nlev, dimid(3))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_def_var(cdfid, "level",
     >                      NF90_FLOAT,dimid(3),varid(3))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(3), 
     >                      "long_name", "hybrid vertical level") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(3), 
     >                      "units", "level") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     rotated pole
      ierror = nf90_def_var(cdfid, "rotated_pole", NF90_CHAR,
     >                      varid=varid(4))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(4), "long_name",
     >     "coordinates of the rotated north pole")
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(4),
     >     "grid_north_pole_longitude", pollon)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid(4),
     >     "grid_north_pole_latitude", pollat)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     End definition mode
      ierror = nf90_enddef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     write longitude, latitude and level
      do i=1,nlon
         val(i) = phymin(1) + real(i-1) *
     >            ( phymax(1) - phymin(1) ) / real(nlon-1)
      enddo
      ierror = nf90_put_var(cdfid, varid(1), val(1:nlon))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      do i=1,nlat
         val(i) = phymin(2) + real(i-1) *
     >            ( phymax(2) - phymin(2) ) / real(nlat-1)
      enddo
      ierror = nf90_put_var(cdfid, varid(2), val(1:nlat))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      do i=1,nlev
         val(i) = real(i) - 1.
      enddo
      ierror = nf90_put_var(cdfid, varid(3), val(1:nlev))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      end


c     -------------------------------------------------------------------
c     putdef
c     -------------------------------------------------------------------

      subroutine putdef_mch (cdfid, newvar, ndimension, misdat, 
     &                      vardim, varmin, varmax, stag, ierror)

      use netcdf
      
      integer        cdfid
      character*(*)  newvar
      integer        ndimension
      real           misdat
      integer        vardim(4)
      real           varmin(4)
      real           varmax(4)
      real           stag(4)
      integer        ierror

      real          eps
      parameter     (eps=0.0001)
      
      integer       ndim
      integer       nvar
      character*80  dimname(100)
      character*80  varname(100)
      integer       dimid
      integer       dimids(4)
      integer       nval
      real          val(1000)
      integer       varid
      integer       varids(4)
      integer       writelon,writelat,writelev,writetim
      character*80  lonname,latname,levname,timname

c     Unit/name table
      character*80  tablefile
      parameter     (tablefile='nil')
      character*80  table_varname (1000)
      character*80  table_longname(1000)
      character*80  table_unit    (1000)
      integer       fid
      integer       ntable

c     Read table information
      if ( tablefile.ne.'nil' ) then
         ntable = 0
         fid    = 10
         open(fid,file=tablefile)
 100      ntable = ntable + 1
          read(fid,*) table_varname (ntable),
     >                        table_longname(ntable),
     >                        table_unit    (ntable)

          print*,table_varname(ntable),table_longname(ntable),
     >             table_unit(ntable)
          
          goto 100
 101     close(fid)
      endif

c     Set the default names for dimensions
      lonname = 'rlon'
      latname = 'rlat'
      levname = 'level'
      timname = 'time'

c     Get number of dimensions and of variables
      ierror = nf90_inquire(cdfid, nDimensions    = ndim,
     >                             nVariables     = nvar )
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Get a list of all dimensions
      if ( ndim.gt.0 ) then
         do i=1,ndim
            ierror = nf90_inquire_dimension(cdfid,i,name = dimname(i))
            IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         enddo
      endif

c     Get a list of all variables
      if ( nvar.gt.0 ) then
         do i=1,nvar
            ierror = nf90_inquire_variable(cdfid,i,name = varname(i))
            IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         enddo
      endif

c     Check whether the new variable name is in conflict with the others
      do i=1,nvar
         if ( newvar.eq.varname(i) ) then
            print*,trim(newvar),' already exists on the file'
            stop
         endif
      enddo

c     Set IDs for dimensions 
      do i=1,4
         dimids(i) = -1
      enddo
      writelon = 0
      writelat = 0
      writelev = 0
      writetim = 0

c     Check whether existing dimensions can be used
      do i=1,ndim

c        Read dimension values
         ierror = nf90_inquire_dimension(cdfid,i,len = nval)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)         
         ierror = NF90_INQ_VARID(cdfid,dimname(i),varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 
         ierror = NF90_GET_VAR(cdfid,varid,val(1:nval))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

c        Check whether it can be used for the first dimension (lon)
         if ( (vardim(1).eq.nval).and.
     >        (abs(varmin(1)-val(1)   ).lt.eps).and.
     >        (abs(varmax(1)-val(nval)).lt.eps) )
     >   then
            dimids(1) = i
         endif

c        Check whether it can be used for the second dimension (lat)
         if ( (vardim(2).eq.nval).and.
     >        (abs(varmin(2)-val(1)   ).lt.eps).and.
     >        (abs(varmax(2)-val(nval)).lt.eps) )
     >   then
            dimids(2) = i
         endif
         
c        Check whether it can be used for the third dimension (lev)
         if ( (vardim(3).eq.nval) ) then
            dimids(3) = i
         endif
         
c        Check whether there is already a time dimension (tim)
         ierror = nf90_inquire(cdfid, unlimitedDimId = dimids(4) )
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

      enddo

c     Decide what dimension names to use
      if ( dimids(1).eq.-1 ) then
         do i=1,ndim
            if ( dimname(i).eq.lonname ) then
               lonname = trim(lonname)//'_'//trim(newvar)
            endif
         enddo
      endif
      if ( dimids(2).eq.-1 ) then
         do i=1,ndim
            if ( dimname(i).eq.latname ) then
               latname = trim(latname)//'_'//trim(newvar)
            endif
         enddo
      endif
      if ( dimids(3).eq.-1 ) then
         do i=1,ndim
            if ( dimname(i).eq.levname ) then
               levname = trim(levname)//'_'//trim(newvar)
            endif
         enddo
      endif

c     Enter definition mode
      ierror = nf90_redef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Define dimensions and corresponding variables if necessary
      if ( dimids(1).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid, lonname, vardim(1), dimids(1))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, lonname,
     >                                 NF90_FLOAT,dimids(1),varids(1))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(1), 
     >        "long_name", "rotated longitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(1), 
     >        "units", "degrees") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(1), 
     >        "standard_name", "longitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         writelon = 1
      endif
      if ( dimids(2).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid, latname, vardim(2), dimids(2))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, latname,
     >                                NF90_FLOAT,dimids(2),varids(2))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(2), 
     >        "long_name", "rotated latitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(2), 
     >        "units", "degrees") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(2), 
     >        "standard_name", "latitude") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         writelat = 1
      endif
      if ( dimids(3).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid, levname, vardim(3), dimids(3))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, levname,
     >                                NF90_FLOAT,dimids(3),varids(3))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(3), 
     >        "long_name", "hybrid vertical level")
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(3), 
     >        "units", "level") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         writelev = 1
      endif
      if ( dimids(4).eq.-1 ) then
         dimid = ndim + 1
         ndim  = ndim + 1
         ierror = nf90_def_dim(cdfid,timname, nf90_unlimited, dimids(4))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_def_var(cdfid, timname,
     >                                NF90_FLOAT,dimids(4),varids(4))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(4), 
     >                 "units", "hours") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_att(cdfid, varids(4), 
     >                 "calendar", "proleptic_gregorian") 
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         
         writetim = 1
      endif

c     Define the variable
      ierror = nf90_def_var(cdfid,newvar,
     >                      NF90_FLOAT,dimids,varid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Write variable attributes
      ierror = nf90_put_att(cdfid, varid, 
     >                 "long_name", "unknown (please add with NCO)") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid, 
     >                 "units", "unknown (please add with NCO)") 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_put_att(cdfid, varid, 
     >                 "_FillValue", real(-999.99) ) 
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)


c     Leave definition mode
      ierror = nf90_enddef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Write dimension arrays
      if ( writelon.eq.1 ) then
         do i=1,vardim(1)
            val(i) = varmin(1) + real(i-1) *
     >                 ( varmax(1) - varmin(1) ) / real(vardim(1)-1)
         enddo
         ierror = nf90_inq_varid(cdfid,lonname,varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_var(cdfid, varid, val(1:vardim(1)))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      endif
      if ( writelat.eq.1 ) then
         do i=1,vardim(2)
            val(i) = varmin(2) + real(i-1) *
     >                 ( varmax(2) - varmin(2) ) / real(vardim(2)-1)
         enddo
         ierror = nf90_inq_varid(cdfid,latname,varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_var(cdfid, varid, val(1:vardim(2)))
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      endif

      if ( writelev.eq.1 ) then
         do i=1,vardim(3)
            val(i) = real(i)
         enddo
         ierror = nf90_inq_varid(cdfid,levname,varid)
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
         ierror = nf90_put_var(cdfid, varid, val(1:vardim(3)) )
         IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      endif

      end


c     -------------------------------------------------------------------
c     puttimes
c     -------------------------------------------------------------------

      subroutine puttimes_mch (cdfid,times,ntimes,ierror)

      use netcdf
      
      integer   cdfid
      real      times(*)
      integer   ntimes
      integer   ierror

      integer   timid
      integer   otimes

c     Get number of times 
      ierror = nf90_inq_dimid(cdfid,'time', timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = nf90_inquire_dimension(cdfid, timid, len = otimes)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      
      if (otimes.lt.ntimes) then
         print *,'Warning: puttimes is extending times-array'
      else if (nfiltim.gt.ntimes) then
         print *,'Warning: puttimes does not cover range of times-array'
      endif

c     Write times
      ierror = NF90_INQ_VARID(cdfid,'time',timid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror = NF90_PUT_VAR(cdfid,timid,times(1:ntimes))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

c     Sync
      ierror = nf90_sync(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end


c     -------------------------------------------------------------------
c     putstart
c     -------------------------------------------------------------------

      subroutine putstart_mch (cdfid,idate,ierror)

      use netcdf

      integer      cdfid
      integer      idate(5)
      integer      ierror

c     Enter definition mode
      ierror = nf90_redef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Write data      
      ierror  = nf90_put_att(cdfid,nf90_global, "starty", idate(1))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "startm", idate(2))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "startd", idate(3))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "starth", idate(4))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)
      ierror  = nf90_put_att(cdfid,nf90_global, "starts", idate(5))
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Exit definition mode
      ierror = nf90_enddef(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Sync
      ierror = nf90_sync(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end


c     -------------------------------------------------------------------
c     cdfwopn
c     -------------------------------------------------------------------

      subroutine cdfwopn_mch (filnam,cdfid,ierror)
      
      use netcdf
      
      character*(*) filnam
      integer       cdfid
      integer       ierror

      ierror = NF90_OPEN(TRIM(filnam),nf90_write, cdfid)
      IF ( ierror /= nf90_NoErr ) PRINT *,NF90_STRERROR(ierror)

      return
      end


c     -------------------------------------------------------------------
c     putdat
c     -------------------------------------------------------------------

      subroutine putdat_mch (cdfid, varnam, time, level, dat, ierror)

      use netcdf
      
      integer        cdfid
      character*(*)  varnam
      real           time
      integer        level
      real           dat(*)
      integer        ierror

      real           eps
      parameter      (eps=0.0001)

      integer        ndims
      real           misdat
      integer        vardim(4)
      real           varmin(4)
      real           varmax(4)
      real           stag(4)
      integer        varid
      integer        ntimes
      real           times(100)
      integer        itime
      integer        corner(4)
      integer        count(4)
      integer        stat
      real,allocatable, dimension (:,:,:,:) :: tmp

c     Get definitions of data
      call getdef_cf (cdfid, trim(varnam), ndims, misdat, 
     &                vardim, varmin, varmax, stag, ierr)
      if (ierr.ne.0)  print *,'*ERROR* in getdef in putdat'

c     Get id of the variable
      ierror = NF90_INQ_VARID(cdfid,varnam,varid)
      IF (ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror)

c     Get a list of all times 
      call gettimes_cf(cdfid,times,ntimes,ierror)

c     Get the time index
      itime = -1
      do i=1,ntimes
         if ( abs(time-times(i)).lt.eps) itime = i
      enddo

c     Start a new time step, if necessary
      if ( itime.eq.-1 ) then
         ntimes        = ntimes + 1
         times(ntimes) = time
         itime         = ntimes
         call puttimes_cf(cdfid,times,ntimes,ierror)
      endif

C     Define data volume to write on the NetCDF file in index space
      corner(1) = 1               
      corner(2) = 1
      count(1)  = vardim(1)       
      count(2)  = vardim(2)
      if (level.eq.0) then
        corner(3) = 1
        count(3)  = vardim(3)
      else
        corner(3) = vardim(3)-level+1
        count(3)  = 1
      endif
      corner(4) = itime
      count(4)  = 1

c     Allocate temporary array
      allocate(tmp(count(1),count(2),count(3),count(4)),stat=stat)

c     Copy data to output array: change orientation of vertical axis
      if ( count(3).gt.1 ) then
         do i=1,count(1)
            do j=1,count(2)
               do k=1,count(3)
                  tmp(i,j,count(3)-k+1,1) =
     >               dat(i+count(1)*(j-1)+count(1)*count(2)*(k-1)) 
               enddo
            enddo
         enddo
      else
         do i=1,count(1)
            do j=1,count(2)
               tmp(i,j,1,1) = dat(i+count(1)*(j-1)) 
            enddo
         enddo
      endif

c    Re-scale specific variables
      if ((trim(varnam) == 'QV') .or. (trim(varnam) == 'QC') .or.
     >    (trim(varnam) == 'QI') .or. (trim(varnam) == 'QR') .or.
     >    (trim(varnam) == 'QS') .or. (trim(varnam) == 'QG') .or.
     >    (trim(varnam) == 'Q_SEDIM') .or.
     >    (trim(varnam) == 'QV_2M')) 
     >then
           tmp = tmp / 1000.
           print*,' WARNING: re-scaling ',trim(varnam),' 1/1000'
      endif 
      if ((trim(varnam) == 'P') .or. (trim(varnam) == 'PS') .or.
     >    (trim(varnam) == 'PMSL') .or. (trim(varnam) == 'P_Z'))
     >then
           tmp = tmp / 0.01
           print*,' WARNING: re-scaling ',trim(varnam),' 1/0.01'
      endif
      if ((trim(varnam) == 'T') .or. (trim(varnam) == 'T_2M') .or.
     >    (trim(varnam) == 'TD_2M') .or.
     >    (trim(varnam) == 'TMIN_2M') .or.
     >    (trim(varnam) == 'TMAX_2M') .or.
     >    (trim(varnam) == 'T_P') .or. (trim(varnam) == 'T_Z'))
     >then
         tmp = tmp + 273.15
         print*,' WARNING: re-scaling ',trim(varnam),' + 273.15'
      endif

c     Write data
      ierror = nf90_put_var(cdfid, varid, tmp,
     >           start = (/ corner(1),corner(2),corner(3),corner(4) /),
     >           count = (/ count (1),count (2),count (3),count (4) /) ) 

c     Deallocate memory
      deallocate(tmp,stat=stat)

c     Sync
      ierror = nf90_sync(cdfid)
      IF(ierror /= nf90_NoErr) PRINT *,NF90_STRERROR(ierror) 

      end

 




 
